#include "image.h"

Image::Image() : QGraphicsView() {

	// Start a timer that calls the drawImg() function on startup whenever the scene is correctly set up
	t = new QTimer;

	this->setStyleSheet("background:transparent; border: none;");

	// These two booleans detect a mouse click that includes dragging
	mousedragged = false;
	buttonpressed = false;

	// This bool can block all the functions (set e.g. for the file-handling dialog, acting like a QDialog.exec())
	blockAllFunctions = false;

	// The array storring the "mouse shortcuts"
	mouseSh.clear();

}

// Called from mainwindow.cpp, setting up a shortcut
void Image::setShortcut(QString key, QString id) {

	// Remove the identifier
	key = key.remove("[M]").trimmed();

	// If no modifier is pressed, we remove the "---"
	if(key.startsWith("---"))
		key = key.remove("---+");

	// And we store the pair in the map
	mouseSh[key] = id;

}

// Zooming the image on ctrl+mouse wheel
void Image::wheelEvent(QWheelEvent *event) {

	if(!blockAllFunctions) {

		// Booleans storring the modifiers state
		bool ctrlPressed = false;
		bool altPressed = false;
		bool shiftPressed = false;

		// Check for modifiers
		if(event->modifiers().testFlag(Qt::ControlModifier))
			ctrlPressed = true;
		if(event->modifiers().testFlag(Qt::AltModifier))
			altPressed = true;
		if(event->modifiers().testFlag(Qt::ShiftModifier))
			shiftPressed = true;

		// Booleans for wheel movement
		bool wheelup = false;
		bool wheeldown = false;

		// Check for wheel movement
		if(event->delta() > 0)
			wheelup = true;
		else
			wheeldown = true;

		// This will hold the key for the map
		QString key = "";

		// Setup the string, first the modifiers
		if(ctrlPressed)
			key = "Ctrl";
		if(altPressed) {
			if(key != "")
				key += "+";
			key += "Alt";
		}
		if(shiftPressed) {
			if(key != "")
				key += "+";
			key += "Shift";
		}

		// If a modifier is pressed, add a "+"
		if(key != "")
			key += "+";

		// Add the wheel movement
		if(wheelup)
			key += "Wheel Up";
		else if(wheeldown)
			key += "Wheel Down";

		// And check for exisiting shortcut
		if(mouseSh.keys().contains(key)) {

			// execute the command
			emit shMouseDo(mouseSh[key], true);

		} else
			QGraphicsView::wheelEvent(event);

	} else
		QGraphicsView::wheelEvent(event);

}

// Detecting all mouse button press events
void Image::mousePressEvent(QMouseEvent *event) {

	// A left button can possibly include dragging -> don't take for shortcut
	if(event->button() == Qt::LeftButton)
		buttonpressed = true;

	QGraphicsView::mousePressEvent(event);

}

// Detect mouse button release events
void Image::mouseReleaseEvent(QMouseEvent *event) {

	if(!blockAllFunctions) {

		// Booleans storring the modifiers state
		bool ctrlPressed = false;
		bool altPressed = false;
		bool shiftPressed = false;

		// Check for modifiers
		if(event->modifiers().testFlag(Qt::ControlModifier))
			ctrlPressed = true;
		if(event->modifiers().testFlag(Qt::AltModifier))
			altPressed = true;
		if(event->modifiers().testFlag(Qt::ShiftModifier))
			shiftPressed = true;

		// Booleans for buttons
		bool doubleClick = false;
		bool rightButton = false;
		bool leftButton = false;
		bool middleButton = false;

		// Check for buttons
		if(event->type() == QMouseEvent::MouseButtonDblClick)
			doubleClick = true;
		else if(event->button() == Qt::LeftButton)
			leftButton = true;
		else if(event->button() == Qt::RightButton)
			rightButton = true;
		else if(event->button() == Qt::MiddleButton)
			middleButton = true;

		// Key string
		QString key = "";

		// Set up the string
		if(ctrlPressed)
			key = "Ctrl";
		if(altPressed) {
			if(key != "")
				key += "+";
			key += "Alt";
		}
		if(shiftPressed) {
			if(key != "")
				key += "+";
			key += "Shift";
		}

		if(key != "")
			key += "+";

		if(doubleClick)
			key += "Double Click";
		else if(rightButton)
			key += "Right Button";
		else if(leftButton)
			key += "Left Button";
		else if(middleButton)
			key += "Middle Button";

		// If mouse click doesn't include dragging -> take it
		if(!mousedragged) {

			// If shortcut exists, execute command
			if(mouseSh.keys().contains(key))
				emit shMouseDo(mouseSh[key],true);
			else if(key == "Left Button")
				emit clicked(event->pos());

		} else
			mousedragged = false;

		// Reset booleans
		buttonpressed = false;

	}

	QGraphicsView::mouseReleaseEvent(event);

}

// Get the current mouse position
void Image::mouseMoveEvent(QMouseEvent *event) {

	if(!blockAllFunctions) {

		emit mousePos(event->pos().x(),event->pos().y());

		// Detect click + dragging
		if(buttonpressed)
			mousedragged = true;

	}

	QGraphicsView::mouseMoveEvent(event);


}

void Image::resizeEvent(QResizeEvent *event) {

	emit resizedTo(this->width(),this->height());

}

