/*############################################

 Found a bug? Leave a comment on qt-apps.org or send me an email:
 photoQt@ymail.com

 I hope you enjoy Photo!

###########################################*/

#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent) {

	finishedLoading = false;

	// Grab the window as early as possible
	QPixmap screen = QPixmap::grabWindow(QApplication::desktop()->winId());
	this->setObjectName("mainwindow");
	this->setStyleSheet("#mainwindow { background: black; }");
	qApp->setQuitOnLastWindowClosed(true);

	// Showing app in fullscreen
	this->showFullScreen();

	// This string holds the current version
	version = "0.8";

	// This is used to get reliable information about the current screen size - We need to use "1" for screenNumber, otherwise it wont load automatically at startup when using multiple monitors...
	desk = QApplication::desktop()->screenGeometry(1);

	// The central widget
	QWidget *central = new QWidget;
	central->setStyleSheet("border: none; background: transparent");
	QVBoxLayout *centralLayout = new QVBoxLayout;
	centralLayout->setMargin(0);
	central->setLayout(centralLayout);
	this->setCentralWidget(central);

	// The settings file
	QFile file(QDir::homePath() + "/.photo/settings");
	QString setFile = "";
	if(file.open(QIODevice::ReadOnly)) {
		QTextStream in(&file);
		setFile = in.readAll();
	} else
		qDebug() << "ERROR! Couldn't open settings file.";

	// If compositing is enabled, then we'll use it (for the mainwindow background)
	if(setFile.contains("Composite=1")) {

		this->setAttribute(Qt::WA_TranslucentBackground);
		central->setStyleSheet("background-color: rgba(0, 0, 0, 190)");

	// If compositing is disabled, then we'll simply take a screenshot and simulate some sort of compositing
	} else {

		// That's the background image
		QPixmap base = screen;
		// We want to overlay it with an half transpatent QPixmap
		QPixmap overlay(desk.height(),desk.width());
		overlay.fill(Qt::transparent);
		QPainter overlayPaint(&overlay);
		overlayPaint.setCompositionMode(QPainter::CompositionMode_SourceOver);
		QColor col;
		col.setRgba(qRgba(0,0,0,190));
		overlayPaint.fillRect(overlay.rect(),col);

		// The finished Pixmap
		QPixmap bg = QPixmap(base.size());
		bg.fill(Qt::transparent);

		// Let's start
		QPainter painter(&bg);
		painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
		painter.drawPixmap(0, 0, base);
		painter.drawPixmap(base.rect(), overlay);

		painter.end();

		// Setting the background image as background of label
		QGraphicsPixmapItem *bgp = new QGraphicsPixmapItem;
		bgp->setPixmap(bg);
		QLabel *bglabel = new QLabel(central);
		bglabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
		bglabel->setFixedSize(desk.width(),desk.height());
		bglabel->setPixmap(bg);
		bglabel->show();
	}

	// Set System App Data
	QApplication::setApplicationName("Photo");
	QApplication::setApplicationVersion(version);
	this->setWindowIcon(QIcon(":/img/logo.png"));

	// Settings regarding the size of the thumbnail view. They have to be set after the settings window but before any of the other widgets!
	thbHeight = 80;
	if(setFile.contains("ThumbnailSize="))
		thbHeight = setFile.split("ThumbnailSize=").at(1).split("\n").at(0).toInt();
	scrollbarheight = 15;

	// Intialise the main Graphicsview
	view = new Image;
	view->setObjectName("view");
	view->setScene(&scene);
	view->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	view->setDragMode(QGraphicsView::ScrollHandDrag);
	view->setMouseTracking(true);
	view->horizontalScrollBar()->setStyleSheet("background: black;");
	view->verticalScrollBar()->setStyleSheet("background: black;");
	connect(view, SIGNAL(shMouseDo(QString,bool)), this, SLOT(shortcutAction(QString,bool)));
	// Disable the scrollbars for settings up window - avoid them from appearing for a split second on startup
	view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	// The Thumbnail view is moved into its own class
	thumbs = new Thumbnails;
	thumbs->height = thbHeight;
	thumbs->setGeometry(0,desk.height()-(thbHeight+scrollbarheight),desk.width(),thbHeight+scrollbarheight);
	thumbs->setFixedHeight(thbHeight+scrollbarheight);
	thumbs->setAlignment(Qt::AlignCenter);
	connect(thumbs, SIGNAL(loadImg(QString,int)), this, SLOT(loadNewImg(QString,int)));

	// After we finished the background, we add the GraphicsViews
	if(setFile.contains("ThumbnailPosition=Top")) {
		centralLayout->addWidget(thumbs);
		centralLayout->addWidget(view);
	} else {
		centralLayout->addWidget(view);
		centralLayout->addWidget(thumbs);
	}

	// The Settings Window (for shortcuts)
	set = new Settings;

	// Setup the tray icon
	setupTrayIcon();

	// Initialize and connect signals of new open dialog (STILL BETA)
	open = new Open(view);
	connect(view, SIGNAL(resizedTo(int,int)), open, SLOT(resizeMe(int,int)));
	connect(open, SIGNAL(loadNewDir(QString)), this, SLOT(loadNewFile(QString)));
	opennav = new OpenNav(thumbs);
	opennav->scrollbarheight = scrollbarheight;
	connect(thumbs, SIGNAL(resizedTo(int,int)), opennav, SLOT(resizeMe(int,int)));
	connect(open, SIGNAL(animating()), opennav, SLOT(animate()));
	connect(opennav->up, SIGNAL(clicked(QString)), open, SLOT(goUp()));
	connect(opennav->home, SIGNAL(clicked(QString)), open, SLOT(goHome()));
	connect(opennav->back, SIGNAL(clicked(QString)), open, SLOT(goBack()));
	connect(opennav->forward, SIGNAL(clicked(QString)), open, SLOT(goForward()));
	connect(open, SIGNAL(enableBack(bool)), opennav->back, SLOT(setEnabled(bool)));
	connect(open, SIGNAL(enableForward(bool)), opennav->forward, SLOT(setEnabled(bool)));
	connect(open, SIGNAL(enableUp(bool)), opennav->up, SLOT(setEnabled(bool)));
	connect(open->close, SIGNAL(clicked()), open, SLOT(animate()));
	connect(open->close, SIGNAL(clicked()), this, SLOT(blockAll()));

	// Set the same version number in the child widgets
	set->version = version;
	set->shortcuts->version = version;

	// Setup the menu on the top right
	m = new Menu(view);
	if(set->thumbnail->thbPosTop->isChecked()) {
		m->menuRectShown = QRect(desk.width()-580,desk.height()-330-scrollbarheight-thumbs->height,530,330);
		m->menuRectHidden = QRect(m->menuRectShown.x(),desk.height(),m->menuRectShown.width(),m->menuRectShown.height());
		m->posAtBottom = false;
	}

	// Setup the exif info widget at the left
	exif = new Exif(view);

	// Setup the about widget and set its version
	about = new About(view, version);

	// The QWidget for file renaming/deleting
	handle = new HandleFile(view);

	// The QWidget for asking for confirmation before rotating/flipping image according to exif data
	exifRotConf = new ExifRotConf(view);

	// Some global variables
	counttot = 0;		// Holds the total amount of pictures in current directory
	countpos = 0;		// The currently viewed picture
	currentfile = "";	// The currently viewed file
	previousfile = "";	// The previously viewed file
	horScaled = 1.0;	// The horizontal scaling factor
	verScaled = 1.0;	// The vertical scaling factor
	zoomSteps = 0;		// How many zoom steps in which direction were made
	rotation = 0;		// The current rotation state
	blockAllFunctions = false;	// This boolean can block all functions (used e.g. for file handling dialog)
	toggleThbTimer = new QTimer;	// Redraw image after thbs are hidden
	skipTrayIcon = false;	// If photo is to be quit despite tray icon, this boolean is set to true

	// Very important, that'll contain the big image
	pix = new GraphicsItem;
	scene.addItem(pix);

	// THE FOLLOWING TWO LABELS ARE PUT INTO THE view LAYOUT
	// This is a little label in the top left, displaying the current position and the total amount of images (eg: 2/143)
	countLabel = new Label(view,"counter");
	countLabel->setText("Photo " + version + "  -  " + tr("Open a file to start."));
	countLabel->adjustSize();
	countLabel->setStyleSheet("background: transparent; border-radius: 10px; border:none;color:white");
	connect(countLabel->dohide, SIGNAL(triggered()), this, SLOT(doHideLabel()));

	// This is a little label in the top (almost) left displaying the current filename
	filenameLabel = new Label(view,"filename");
	filenameLabel->adjustSize();
	filenameLabel->setStyleSheet("background: transparent; border-radius: 10px; border:none;color:white");
	connect(filenameLabel->dohide, SIGNAL(triggered()), this, SLOT(doHideLabel()));

	// A modified label (with clicked SIGNAL) to display a "close photo" button
	doClose = new Label(view,"close");
	doClose->setText("<b><span style=\"color: white\">x</span></b>");
	doClose->setParent(this);
	doClose->setToolTip(tr("Close Photo"));
	doClose->setFixedSize(15,15);
	doClose->show();
	connect(doClose, SIGNAL(clicked()), this, SLOT(close()));
	connect(doClose->dohide, SIGNAL(triggered()), this, SLOT(doHideLabel()));

	// The view layout that contains the label layout for the two labels above
	QVBoxLayout *viewLay = new QVBoxLayout;
	QHBoxLayout *labLay = new QHBoxLayout;
	// If the thumbnail bar is on top, the label is shown at the bottom right
	if(set->thumbnail->thbPosTop->isChecked()) {
		labLay->addWidget(filenameLabel);
		labLay->addWidget(countLabel);
		labLay->addStretch();
		labLay->addWidget(doClose);
		viewLay->addStretch();
		viewLay->addLayout(labLay);
	// Otherwise at the top left
	} else {
		labLay->addWidget(countLabel);
		labLay->addWidget(filenameLabel);
		labLay->addStretch();
		labLay->addWidget(doClose);
		viewLay->addLayout(labLay);
		viewLay->addStretch();
	}
	view->setLayout(viewLay);

	// This is a timer, that updates a file every 500 milliseconds with the current timestamp, and it checks a different file for possible commands
	globalTimer = new QTimer;
	globalTimer->setInterval(500);
	connect(globalTimer, SIGNAL(timeout()), this, SLOT(globalTimerDO()));
	globalTimer->start();

	// Load the Shortcuts
	loadShortcuts();

	//////////////////////////////
	////////// General SIGNAL/SLOTS
	//////////////////////////////

	// Change of Shortcuts
	connect(set->shortcuts, SIGNAL(updateShortcuts()), this, SLOT(loadShortcuts()));
	// When the settings window is closed, make sure, that this window gets the focus again
	connect(set->shortcuts, SIGNAL(focusParentWindow()), this, SLOT(focusThisWindow()));
	connect(set, SIGNAL(settingsSaved()), this, SLOT(afterSettingsClosed()));

	// Enable zooming
	connect(view, SIGNAL(zoom(int)), this, SLOT(zoom(int)));
	// The drawImg() function is called whenever the view is correctly set up
	connect(view->t, SIGNAL(timeout()), this, SLOT(doDrawImgStartUp()));
	connect(view, SIGNAL(mousePos(int,int)), m, SLOT(animateMenu(int,int)));
	connect(view, SIGNAL(mousePos(int,int)), exif, SLOT(animate(int,int)));
	connect(view, SIGNAL(clicked(QPoint)), this, SLOT(getSceneClick(QPoint)));

	// Click on a menu item
	connect(m, SIGNAL(itemClicked(QString,int)), this, SLOT(menuClicked(QString,int)));

	// file handling
	connect(handle, SIGNAL(fileRenamed(QString)), this, SLOT(doRename(QString)));
	connect(handle, SIGNAL(fileDeleted()), this, SLOT(doDelete()));
	connect(handle, SIGNAL(blockAll(bool)), this, SLOT(blockAll(bool)));

	// About widget
	connect(about, SIGNAL(block(bool)), this, SLOT(blockAll(bool)));

	// If the exif data suggest a rotating/flipping of the image
	connect(exif, SIGNAL(setOrientation(int)), this, SLOT(setExifOrientation(int)));
	// exifRotConf asks the user for confirmation before rotating/flipping if not disabled
	connect(exifRotConf, SIGNAL(rotate(int,bool)), this, SLOT(setExifOrientation(int,bool)));
	connect(exifRotConf, SIGNAL(block(bool)), this, SLOT(blockAll(bool)));
	connect(exifRotConf, SIGNAL(rotDontAsk(bool)), this, SLOT(setExifOrientNeverAsk(bool)));

	// And at the end re-enable the scrollbars
	view->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	view->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

	// Apply all the settings to the current instance
	afterSettingsClosed();

	// Adjust zoom after rotation
	zoomAfterRot = new QTimer;
	zoomAfterRot->setSingleShot(true);
	connect(zoomAfterRot, SIGNAL(timeout()), this, SLOT(timerResetZoom()));

	connect(toggleThbTimer, SIGNAL(timeout()), this, SLOT(toggleTimer()));

	finishedLoading = true;

}

// Do some stuff whenever the settings have been saved
void MainWindow::afterSettingsClosed() {

	// Adjust the menu sensitivity
	m->xSensitivity = set->interface->menu->value()*3;
	m->ySensitivity = set->interface->menu->value()*10;

	// Show/Hide the "x" for closing (other two labels managed by updateCounter())
	if(set->interface->hideX->isChecked())
		doClose->hide();
	else
		doClose->show();

	// Set if cache dis-/enabled
	if(set->thumbnail->thumbCache->isChecked())
		thumbs->cacheEnabled = true;
	else
		thumbs->cacheEnabled = false;

	if(set->interface->trayUse->isChecked())
		trayIcon->show();
	else
		trayIcon->hide();

	// Set the type of cache to be used
	if(set->thumbnail->cacheFile->isChecked())
		thumbs->thread->type = "files";
	else
		thumbs->thread->type = "database";

	// Adjust the exif widget
	for(int i = 0; i < set->exif->allItemsShort.length(); ++i) {
		if(set->exif->allItemsShort.at(i) != "") {
			exif->showItems[set->exif->allItemsShort.at(i)] = set->exif->allCheckBoxes[set->exif->allItemsShort.at(i)]->isChecked();
			if(currentfile != "")
				exif->items[set->exif->allItemsShort.at(i)]->setVisible(set->exif->allCheckBoxes[set->exif->allItemsShort.at(i)]->isChecked());
		}
	}

	// Set diffferent exif widget settings
	exif->mouseTrickerEnable = !set->exif->exifShowsOnMouse->isChecked();
	exif->stay->setVisible(!set->exif->exifShowsOnMouse->isChecked() && currentfile != "");
	exif->adjustHeight();
	exif->onlineservice = set->exif->exifGPSBrowser->selectedItems().at(0)->text();

	// Adjust the transition effect
	pix->transitionSetChange(set->interface->transition->value());

	// Update the counter (and possibly hiding text labels)
	updateCounter();

}

// Block all functions (e.g. used for file handling
void MainWindow::blockAll(bool block) {

	blockAllFunctions = block;
	view->blockAllFunctions = block;
	m->blockAllFunctions = block;
	thumbs->blockAllFunction = block;
	exif->blockAll = block;

	if(open->oShown) {
		countLabel->hide();
		filenameLabel->hide();
		doClose->hide();
	} else {
		countLabel->show();
		filenameLabel->show();
		doClose->show();
	}


	thumbs->horizontalScrollBar()->setVisible(!block);

}

// The close event cleans up
void MainWindow::closeEvent(QCloseEvent *e) {

	qDebug() << "Skip Tray:" << skipTrayIcon;

	if(set->interface->trayUse->isChecked() && !skipTrayIcon) {
		this->hide();
		qDebug() << "Hiding to System Tray.";
		e->ignore();
	} else {
		scene.clear();
		thumbs->scene.clear();
		thumbs->thread->terminate();
		open->openThread->terminate();

		// Ensure all windows are closed on exit
		if(set->isVisible())
			set->close();
		if(set->shortcuts->key->isVisible())
			set->shortcuts->key->close();

		e->accept();

		qDebug() << "Goodbye.";
	}

}

// Delete file and update view
void MainWindow::doDelete() {

	qDebug() << "Do Delete";

	QFile file(currentfile);
	if(file.remove()) {

		if(countpos == counttot-1)
			getPrev();
		else {
			getNext();
			countpos -= 1;
		}

		counttot -= 1;
		thumbs->allimgs.removeAt(countpos);
		thumbs->allThumbs.removeAt(countpos);
		if(counttot == 0) {
			pix->setPixmap(QPixmap());
			currentfile = "";
			thumbs->currentfile = "";
			thumbs->scene.clear();
			for(int i = 0; i < exif->labelsId.length(); ++i) {
				if(exif->labelsId.at(i) != "")
					exif->items[exif->labelsId.at(i)]->hide();
			}
			exif->empty->show();
			exif->stay->hide();
			countLabel->setText("Photo " + version + "  -  " + tr("Open a file to start."));
			filenameLabel->setText("");
			openFile();
		} else {
			thumbs->allImgs();
			updateCounter();
		}

	}

}

// On startup, the graphicsview check with a timer when the scene is set up correctly and then calls the draw function
void MainWindow::doDrawImgStartUp() {

	if(view->height() > desk.height()-thbHeight-scrollbarheight-50 && view->width() > desk.width()-20) {
		drawImg();
		view->t->stop();
	}

}

// When a user hides a label by right clicking on it
void MainWindow::doHideLabel() {

	// Get the sender object name
	QString obj = ((Label *) sender())->objectName();

	// Hide the label clicked upon:

	if(obj == "close") {
		set->interface->hideX->setChecked(true);
		set->saveSettings();
	} else if(obj == "counter") {
		set->interface->hideCounter->setChecked(true);
		set->saveSettings();
	} else if(obj == "filename") {
		set->interface->hideFilename->setChecked(true);
		set->saveSettings();
	}

}

// After a file renaming, the current view is updated
void MainWindow::doRename(QString newName) {

	thumbs->allThumbs.replace(countpos,newName);

	QFileInfo info(newName);

	thumbs->allimgs.replace(countpos,info);

	thumbs->allPixItem.at(countpos)->setToolTip("<span style=\"color: white; background-color: black; white-space: nowrap\">" + newName + "." + thumbs->allimgs.at(countpos).completeSuffix() + "</span>");
	thumbs->allPixItem.at(countpos)->setData(123,newName);

	currentfile = newName;
	thumbs->currentfile = newName;
	thumbs->allImgs();
	countpos = thumbs->allimgs.indexOf(currentfile);
	updateCounter();

	qDebug() << "New filename:" << newName;

}

// Displaying the big image
// Possible dothis values: normal, zoomin, zoomout, ignorethumbnails, justreplacepix, noexifupdate
void MainWindow::drawImg(QString dothis) {

	if(!blockAllFunctions) {

		// The settings are reloaded when the thumbnail size changed
		if(thbHeight != set->thumbnail->thumbSizeSlider->value())
			loadSettings();

		// If no file was passed on to photo
		if(currentfile == "") {

			// ... open one!
			openFileOld();

		} else {

			// Update the rotation states (needed for transitioning)
			pix->prevRotated = pix->curRotated;
			if(whichIsRotated.value(thumbs->allimgs.indexOf(currentfile)) != 0)
				pix->curRotated = true;
			else
				pix->curRotated = false;

			// Reset a possible rotation
			if(rotation != 0 && dothis != "exifredraw") {
				view->rotate(-rotation);
				rotation = 0;
			}

			qDebug() << "Drawing image:" << currentfile;

			// At first function call: Collect general data
			if(counttot == 0) {
				thumbs->height = thbHeight;
				thumbs->cacheEnabled = set->thumbnail->thumbCache->isChecked();
				thumbs->allImgs();
				counttot = thumbs->counttot;
			}

			// If there is only one image in the directory at all, then the thumbnail view isnot needed and hidden
			if(counttot == 1 && thumbs->isVisible() && dothis != "ignorethumbnails") {
				toggleThumbnails(false);
			}

			// Scaling and adding the image to display (using QImageReader for better performance)
			QImageReader reader(currentfile);

			// Save the file format straight away for later use
			QString fileformat = reader.format().toLower();

			// Store the width/height for later use
			int readerWidth = reader.size().width();
			int readerHeight = reader.size().height();

			// Store the original size (used for the exif info)
			QSize origSize = QSize(readerWidth,readerHeight);

			// If the image was zoomed in and is now bigger than its actual size, then don't do anything, just replace the image
			if(horScaled > 1 && verScaled > 1)
				dothis = "justreplacepix";
			// If the image was zoomed out, then we take the normal size and scale the view down
			else if(horScaled < 1 && verScaled < 1)
				dothis = "normal";
			// If the view is normal, just reset the matrixes (just to be on the safe site)
			else if(zoomSteps == 0 && dothis != "exifredraw") {
				view->resetTransform();
				view->resetMatrix();
			}

			// That determines the margin the big image has on the top and the bottom
			int border = 5;

			// If the image was zoomed in (but not yet beyond its actual size)
			if(dothis == "zoomin") {

				// Determine the factor by which the view has to be scaled
				int compareHeight1 = this->height();
				if(reader.size().height() < view->height()-border)
					compareHeight1 = reader.size().height();
				int compareHeight2 = view->height()-border;
				if(reader.size().height() < compareHeight2)
					compareHeight2 = reader.size().height();
				int compareWidth = view->width()-border;
				if(reader.size().width() < compareWidth)
					compareWidth = reader.size().width();


				float q1 = verScaled*(compareHeight1-thbHeight-scrollbarheight)/(readerHeight*1.0);
				if(compareHeight2 > compareHeight1-thbHeight-scrollbarheight-50)
					q1 = verScaled*compareHeight2/(readerHeight*1.0);
				float q2 = horScaled*compareWidth/(readerWidth*1.0);

				float q = q1;
				if(q1 > q2)
					q = q2;

				view->scale(q,q);

			// if the image was either zoomed out or not zoomed at all
			} else if(dothis != "justreplacepix" && dothis != "nextinanimation" && dothis != "zoomactual") {

				// If not zoomed at all
				if(dothis != "normal") {
					view->resetTransform();
					view->resetMatrix();
				}

				if(readerWidth > desk.width()-10) {
						float q = (desk.width()-10)/(readerWidth*1.0);
						readerWidth *= q;
						readerHeight *= q;
				}
				int thb = thbHeight+scrollbarheight;
				if(!thumbs->isVisible())
					thb = 0;
				if(readerHeight > desk.height()-thb-2*border) {
					float q = (this->height()-thb-2*border)/(readerHeight*1.0);
					readerWidth *= q;
					readerHeight *= q;
				}

				reader.setScaledSize(QSize(readerWidth,readerHeight));

			}

			// Eventually load the image
			QImage img = reader.read();

			// If the image is animated, set it as movie
			if(reader.supportsAnimation() && reader.imageCount() > 1)
				pix->setMovie(currentfile,readerWidth,readerHeight);
			// Otherwise do the normal setPixmap()
			else {

				if(dothis != "zoomin" && dothis != "zoomout" && dothis != "zoomreset" && dothis != "ignorethumbnails" && zoomSteps == 0)
					pix->setPixmap(QPixmap::fromImage(img));
				else
					pix->setPixmap(QPixmap::fromImage(img),false,false);
			}

			if(zoomSteps == 0)
				// Adjust the position
				pix->setPos(view->mapToScene((desk.width()-pix->pixmap().width())/2.0,(view->height()-pix->pixmap().height())/2.0));

			// These formats known by Photo are supported by exiv2
			QStringList formats;
			formats << "bmp" << "gif" << "tiff" << "jpg" << "jpeg" << "png";

			// Update the current position of the image in the directory
			countpos = thumbs->allimgs.indexOf(currentfile);

			// If image previously was rotated, rotate again
			if(whichIsRotated.value(countpos) != 0) {
				pix->curRotated = true;
				rotation = whichIsRotated.value(countpos);
				view->rotate(rotation);
			}

			// If image previously was flipped, flip again
			if(whichIsFlippedHor.value(countpos))
				view->scale(-1,1);
			if(whichIsFlippedVert.value(countpos))
				view->scale(1,-1);

			// Adjust scene rect
			scene.setSceneRect(scene.itemsBoundingRect());

			// Update the counter top left
			updateCounter();

			// If image changed, update exif data
			if(dothis != "exifredraw" && pix->data(Qt::UserRole).toString() != currentfile) {

				pix->setData(Qt::UserRole,currentfile);

				// If supported, load exiv2 data
				if(formats.contains(fileformat))
					// Update the exif data widget
					exif->updateData(currentfile,origSize);
				// Otherwise set exiv2 to "unsupported"
				else
					// Update the exif data widget
					exif->updateData(currentfile,origSize,false);

			}

			// If image is loaded in "to fit" mode, possibly adjust the size
			if(zoomSteps == 0 && dothis != "zoomin" && dothis != "zoomout" && dothis != "zoomactual") {
				if(pix->pixmap().width() > pix->pixmap().height() && pix->pixmap().width() > view->height()) {
					if(abs(rotation) == 90 || abs(rotation) == 270)
						view->scale(float(view->height())/float(pix->pixmap().width()),float(view->height())/float(pix->pixmap().width()));
					else if(pix->pixmap().height() > view->height())
						view->scale(float(pix->pixmap().width())/float(view->height()),float(pix->pixmap().width())/float(view->height()));
				}
			}

		}

	} else
		qDebug() << "Sorry, blocked...";

}

// A shortcut was pressed
void MainWindow::executeShortcut() {

	// Get the key that was pressed
	QShortcut *keySH = (QShortcut *) sender();
	QString key = mapAllSh[keySH->key().toString()];

	qDebug() << key;

	if(!blockAllFunctions) {

		// In case something went wrong (shouldn't happen)
		if(key.split(":::::").length() == 2) {

			// The first part contains the close setting (1=closePhoto)
			int close = key.split(":::::").at(0).toInt();
			// The second part is the actual command
			key = key.split(":::::").at(1);

			// Internal command start with two "_"
			if(key.startsWith("__")) {

				shortcutAction(key);


			// All external commands
			} else {

				QProcess *p = new QProcess;
				key = key.replace("%f",'"' + currentfile + '"');
				key = key.replace("%d",'"' + QFileInfo(currentfile).absoluteDir().absolutePath() + '"');
				qDebug() << "KEY:" << key;
				p->start(key);
				qDebug() << close;
				while(!p->waitForStarted()) { }
				if(close == 1)
					this->close();

			}

		}


	// If functions are blocked because file handle dialog is shown
	} else if(handle->isShown) {

		// An "escape" closes file dialog
		if(keySH->key().toString() == QKeySequence(Qt::Key_Escape).toString())
			handle->animate();
		// A "enter" or "return" accepts and closes widget
		else if(keySH->key().toString() == QKeySequence(Qt::Key_Return).toString() || keySH->key().toString() == QKeySequence(Qt::Key_Enter).toString()) {
			if(handle->layout()->objectName() == "rename")
				handle->renameDoIt();
			else
				handle->deleteDoIt();
		}

	// If funtions are blocked because About widget is shown
	} else if(about->isShown) {

		// "escape" and shortcut set for it closes it
		if(keySH->key().toString() == QKeySequence(Qt::Key_Escape) || key.contains("__about"))
			about->animate();

	// If functions are blocked because exifconfirm is shown
	} else if(exifRotConf->isShown) {

		// "escape" closes it
		if(keySH->key().toString() == QKeySequence(Qt::Key_Escape).toString())
			exifRotConf->animate();
		// Confirmed on Enter/Return
		if(keySH->key().toString() == QKeySequence(Qt::Key_Enter).toString() || keySH->key().toString() == QKeySequence(Qt::Key_Return).toString())
			exifRotConf->ok->click();
	} else if(open->oShown) {

		// "escape" closes it
		if(keySH->key().toString() == QKeySequence(Qt::Key_Escape) || key.contains("__open")) {
			open->animate();
			emit blockAll(false);
		}

	}

}

// This function flips the image horizontally and vertically
void MainWindow::flip(int i, bool ignoreblock) {

	if(!blockAllFunctions || ignoreblock) {

		if(currentfile != "") {

			qDebug() << "Flipping image";

			// Any change here disables a possible change issued by exif data
			manuallyRotated[countpos] = true;

			if(i == 1) {

				// Flip horizontally
				view->scale(-1,1);

				// And update data
				if(whichIsFlippedHor.value(countpos) == true) {
					whichIsFlippedHor.remove(countpos);
					whichIsFlippedHor.insert(countpos,false);
				} else {
					whichIsFlippedHor.remove(countpos);
					whichIsFlippedHor.insert(countpos,true);
				}

			} else if(i == 2) {

				// Flip vertically
				view->scale(1,-1);

				// And update data
				if(whichIsFlippedVert.value(countpos) == true) {
					whichIsFlippedVert.remove(countpos);
					whichIsFlippedVert.insert(countpos,false);
				} else {
					whichIsFlippedVert.remove(countpos);
					whichIsFlippedVert.insert(countpos,true);
				}

			}
		}

	} else
		qDebug() << "Sorry, blocked...";

}

// Set the focus to this window
void MainWindow::focusThisWindow() {
	this->activateWindow();
	this->setFocus();
}

// Load the next image in the directory (if there is one)
void MainWindow::getNext() {

	if(!blockAllFunctions && currentfile != "") {

		if(countpos < counttot-1) {
			qDebug() << "Getting next image";
			++countpos;
			previousfile = currentfile;
			currentfile = thumbs->allimgs.at(countpos).absoluteFilePath();
			thumbs->currentfile = currentfile;
			drawImg();
			thumbs->updImgMark(thumbs->allimgs.indexOf(previousfile),thumbs->allimgs.indexOf(currentfile));
		} else if(set->interface->loop->isChecked()) {
			qDebug() << "No next image -> jumping to first";
			thumbs->gotoFirstLast("first");
		} else
			qDebug() << "No next image available";

	} else if(currentfile != "")
		qDebug() << "Sorry, blocked...";

}

// Load the previous image in the directory (if there is one)
void MainWindow::getPrev() {

	if(!blockAllFunctions && currentfile != "") {

		if(countpos > 0) {
			qDebug() << "Getting previous image";
			--countpos;
			previousfile = currentfile;
			currentfile = thumbs->allimgs.at(countpos).absoluteFilePath();
			thumbs->currentfile = currentfile;
			drawImg();
			thumbs->updImgMark(thumbs->allimgs.indexOf(previousfile),thumbs->allimgs.indexOf(currentfile));
		} else if(set->interface->loop->isChecked()) {
		       qDebug() << "No previous image -> jumping to last";
		       thumbs->gotoFirstLast("last");
		} else
			qDebug() << "No previous image available";

	} else if(currentfile != "")
		qDebug() << "Sorry, blocked...";

}

// Get the position of the mouse click on the scene -- only if no shortcut was set for simple mouse click!!
void MainWindow::getSceneClick(QPoint p) {

	if(!blockAllFunctions) {

		QPointF point = view->mapToScene(p);

		int x = point.x();
		int y = point.y();

		int xg = pix->pos().x();
		int yg = pix->pos().y();
		int wg = pix->pixmap().width();
		int hg = pix->pixmap().height();

		// If click was on grey area outside image and corresponding option is set, then close Photo
		if(x < xg-20 || x > xg+wg+20 || y < yg-20 || y > yg+hg+20 || currentfile.trimmed().length() == 0) {
			if(set->interface->grey->isChecked()) {
				skipTrayIcon = true;
				this->close();
			}
		}

	} else {

		if(about->isShown)
			shortcutAction("__about");
		else
			qDebug() << "Sorry, blocked...";

	}

}

// Ensures only one instance is running and checks for command line arguments
void MainWindow::globalTimerDO() {

	desk = view->geometry();
	QRect tmpShown = QRect((desk.width()-900)/2.0,(desk.height()+thumbs->height-550)/2.0,900,550);
	QRect tmpHidden = QRect(about->rectShown.x(),-about->rectShown.height(),about->rectShown.width(),about->rectShown.height());

	if(tmpShown != about->rectShown || tmpHidden != about->rectHidden) {
		// The current geometry
		m->viewHeight = thumbs->height;

		// Adjust menu position depending on location at top or bottom edge
		if(set->thumbnail->thbPosTop->isChecked()) {
			m->menuRectShown = QRect(desk.width()-580,desk.height()-330,530,330);
			m->menuRectHidden = QRect(m->menuRectShown.x(),desk.height(),m->menuRectShown.width(),m->menuRectShown.height());
			m->posAtBottom = false;
		} else {
			m->menuRectShown = QRect(desk.width()-580,0,530,330);
			m->menuRectHidden = QRect(m->menuRectShown.x(),-m->menuRectShown.height(),m->menuRectShown.width(),m->menuRectShown.height());
			m->posAtBottom = true;
		}

		// And set geometry
		if(m->mShown)
			m->setGeometry(m->menuRectShown);
		else
			m->setGeometry(m->menuRectHidden);


		// Adjust geometry of about widget
		about->rectShown = QRect((desk.width()-900)/2.0,(desk.height()+thumbs->height-550)/2.0,900,550);
		about->rectHidden = QRect(about->rectShown.x(),-about->rectShown.height(),about->rectShown.width(),about->rectShown.height());
		if(about->isShown)
			about->setGeometry(about->rectShown);
		else
			about->setGeometry(about->rectHidden);
	}


	// Updates the "running" file with current timestamp
	QFile t(QDir::homePath() + "/.photo/running");
	if(t.open(QIODevice::WriteOnly)) {
		QTextStream out(&t);
		out << QDateTime::currentMSecsSinceEpoch();
		t.close();
	} else
		qDebug() << "ERROR! Unable to write to file '~/.photo/running' - unable to control this instance through command line.";

	// Checks the "cmd" file for commands to be executed
	QFile cmd(QDir::homePath() + "/.photo/cmd");
	QString all = "";
	if(cmd.open(QIODevice::ReadWrite)) {
		QTextStream in(&cmd);
		all = in.readAll();
		cmd.close();
	} else
		qDebug() << "ERROR! Can't read '~/.photo/cmd'.";

	// Remove file if command is read in
	if(all != "")
		cmd.remove();

	// This boolean is set to true if a command line option was used, and left at false if filename is submitted
	bool done = false;

	if(all.contains("toggle")) {
		qDebug() << "Got command: toggle";
		if(this->isVisible() && set->interface->trayUse->isChecked())
			this->hide();
		else
			this->showFullScreen();
		this->raise();
		done = true;
	}
	if(all.contains("show")) {
		qDebug() << "Got command: show";
		if(this->isHidden())
			this->showFullScreen();
		this->raise();
		done = true;
	}
	if(all.contains("open")) {
		qDebug() << "Got command: open";
		if(this->isHidden())
			this->showFullScreen();
		this->raise();
		openFileOld();
		done = true;
	}
	if(all.contains("hide")) {
		qDebug() << "Got command: hide";
		if(this->isVisible() && set->interface->trayUse->isChecked())
			this->hide();
		done = true;
	}

	if(done == false && all.trimmed() != "") {

		qDebug() << "Got command: filename";

		if(this->isHidden())
			this->showFullScreen();

		this->raise();

		whichIsRotated.clear();
		whichIsFlippedVert.clear();
		whichIsFlippedHor.clear();
		manuallyRotated.clear();

		// If thumbnail size was changed -> reload thumbnail settings
		if(thbHeight != set->thumbnail->thumbSizeSlider->value())
			loadSettings();

		// Reset all variables
		currentfile = all.trimmed();
		thumbs->currentfile = currentfile;
		thumbs->cacheEnabled = set->thumbnail->thumbCache->isChecked();
		thumbs->allimgs.clear();
		thumbs->allPixHov.clear();
		thumbs->allPixImg.clear();
		thumbs->allPixItem.clear();
		thumbs->allPixSel.clear();

		// And load the image and the directory
		thumbs->allImgs();
		qDebug() << "Cache enabled:" << thumbs->cacheEnabled;
		countpos = thumbs->allimgs.indexOf(currentfile);
		counttot = thumbs->counttot;
		drawImg();
		updateCounter();

	}

}

// File Handling
void MainWindow::handleFile(int todo) {

	if(!blockAllFunctions) {

		if(currentfile != "") {

			// if the menu is visible, animate it out
			if(m->mShown)
				m->animateMenu(0,0,true);
			if(exif->isShown && !exif->staying)
				exif->animate(0,0,true,false);

			// Rename the current file
			if(todo == 1) {

				qDebug() << "Rename File?";

				handle->currentfile = currentfile;
				handle->animate("rename");

			// Delete the current file
			} else if(todo == 2) {
				qDebug() << "Delete File?";

				handle->currentfile = currentfile;
				handle->animate("delete");

			// Make a copy at a different location of current file
			} else if(todo == 3) {

				qDebug() << "Copy File?";

				QString dir = QFileDialog::getSaveFileName(this,tr("Copy file to..."),thumbs->allimgs.at(countpos).absoluteFilePath(),tr("Image") + " (*." + thumbs->allimgs.at(countpos).completeSuffix() + ")");

				if(dir != "" && dir != currentfile) {

					QFile file(dir);

					// If the new file already exists, remove it (a warning happens already in the QFileDialog)
					if(file.exists())
						file.remove();

					if(!file.copy(currentfile,dir))
						qDebug() << "ERROR: Couldn't copy file.";
					else {

						// If new file is in same directory as currently loaded -> update view
						if(QFileInfo(dir).absolutePath() == QFileInfo(currentfile).absolutePath()) {

							thumbs->allImgs();
							currentfile = dir;
							thumbs->currentfile = dir;
							counttot = thumbs->allimgs.length();
							countpos = thumbs->allimgs.indexOf(dir);
							updateCounter();
							if(!thumbs->isVisible() && counttot == 2)
								toggleThumbnails();

						}
					}
				}

			// Move file to a new location
			} else if(todo == 4) {

				qDebug() << "Move File?";

				QString dir = QFileDialog::getSaveFileName(this,tr("Move file to..."),thumbs->allimgs.at(countpos).absoluteFilePath(),tr("Image") + " (*." + thumbs->allimgs.at(countpos).completeSuffix() + ")");

				if(dir != "" && dir != currentfile) {

					// If new file location is in same directory, but only the name changed, then handle it as a simple rename
					if(QFileInfo(dir).absolutePath() == QFileInfo(currentfile).absolutePath() && dir != currentfile) {

						QFile file(currentfile);
						if(file.copy(dir)) {
							if(!file.remove())
								qDebug() << "ERROR: Couldn't remove old version of file! The file exists now twice!";
							doRename(dir);
						} else
							qDebug() << "ERROR: Couldn't move file to new location!!";

					// Moving file to a different directory
					} else {
						QFile file(dir);
						if(file.exists())
							file.remove();
						if(!file.copy(currentfile,dir))
							qDebug() << "ERROR: Couldn't copy file.";
						doDelete();
					}
				}

			}

		}

	} else
		qDebug() << "Sorry, blocked...";

}

// Load a new file
void MainWindow::loadNewFile(QString file) {

	// Stopping a possibly running thread
	thumbs->thread->terminate();
	open->openThread->terminate();

	qDebug() << "Loading new file.";

	emit blockAll(false);

	// Selected image is in same folder as previously loaded, don't reload thbs
	if(QFileInfo(file).absolutePath() == QFileInfo(currentfile).absolutePath()) {

		countpos = thumbs->allimgs.indexOf(file);
		previousfile = currentfile;
		currentfile = thumbs->allimgs.at(countpos).absoluteFilePath();
		thumbs->currentfile = currentfile;
		drawImg();
		thumbs->updImgMark(thumbs->allimgs.indexOf(previousfile),thumbs->allimgs.indexOf(currentfile));

	// Load a new folder
	} else {

		// Reset some global QMaps
		whichIsRotated.clear();
		whichIsFlippedVert.clear();
		whichIsFlippedHor.clear();
		manuallyRotated.clear();

		// If thumbnail size was changed -> reload thumbnail settings
		if(thbHeight != set->thumbnail->thumbSizeSlider->value())
			loadSettings();

		// Reset all variables
		currentfile = file;
		thumbs->currentfile = currentfile;
		thumbs->cacheEnabled = set->thumbnail->thumbCache->isChecked();
		thumbs->allimgs.clear();
		thumbs->allPixHov.clear();
		thumbs->allPixImg.clear();
		thumbs->allPixItem.clear();
		thumbs->allPixSel.clear();

		// And load the image and the directory
		thumbs->allImgs();
		qDebug() << "Cache enabled:" << thumbs->cacheEnabled;
		countpos = thumbs->allimgs.indexOf(currentfile);
		counttot = thumbs->counttot;
		drawImg();

	}

	// Update the labels at the upper edge of photo
	updateCounter();

}

// After a click on a thumbnail image
void MainWindow::loadNewImg(QString str, int pos) {

	if(!blockAllFunctions) {

		qDebug() << "Click on Thumbnail image";

		if(str.trimmed() != "") {
			previousfile = currentfile;
			currentfile = str;
			thumbs->currentfile = currentfile;
			countpos = pos;
			drawImg();
		}

	} else
		qDebug() << "Sorry, blocked...";

}

// Load the thumbnail settings (the thumbnail size can be changed in settings during runtime)
void MainWindow::loadSettings() {

	thbHeight = set->thumbnail->thumbSizeSlider->value();
	thumbs->height = thbHeight;
	thumbs->setGeometry(0,desk.height()-(thbHeight+scrollbarheight),desk.width(),thbHeight+scrollbarheight);
	thumbs->setFixedHeight(thbHeight+scrollbarheight);
	view->setFixedHeight(desk.height()-(thbHeight+scrollbarheight));

}

// Load the shortcuts (from file)
void MainWindow::loadShortcuts() {

	QFile file(QDir::homePath() + "/.photo/shortcuts");

	if(!file.open(QIODevice::ReadOnly))
		qDebug() << "ERROR: The shortcuts file couldn't be read! No shortcuts will be available!";
	else {

		// Get the file content
		QTextStream in(&file);
		QStringList all = in.readAll().split("\n");

		// currentSH holds all the once setup shortcuts
		QStringList currentSH;

		view->mouseSh.clear();

		for(int i = 0; i < all.length(); ++i) {

			// The first line holds the version -> ignore it
			if(!all.at(i).startsWith("Version=") && all.at(i).trimmed() != "") {

				// Get the values
				QString close = all.at(i).split("::").at(0);
				QString shortcut = all.at(i).split("::").at(1);
				QString command = all.at(i).split("::").at(2);

				// If the shortcut starts with a "[M]", then it's a mouse shortcut -> set in image.cpp
				if(shortcut.startsWith("[M]")) {

					view->setShortcut(shortcut,command);

				} else {


					// The first line holds the version -> ignore it
					if(!all.at(i).startsWith("Version=") && all.at(i).trimmed() != "") {

						// If the shortcut existed already and/or has changed, or if it was first deleted and then reenabled (in the shortcut management), then we just adjust the command, and if their enabled or disabled (this way, any change in the shortcuts settings will take immediate effect!)
						if(mapAllSh.keys().contains(QKeySequence(shortcut).toString())) {
							mapAllSh[QKeySequence(shortcut).toString()] = QString("%1:::::%2").arg(close).arg(command);
							listAllSh[QKeySequence(shortcut).toString()]->setEnabled(true);
							currentSH << QKeySequence(shortcut).toString();
						} else {
							QShortcut *dyn = new QShortcut(QKeySequence(shortcut),this);
							connect(dyn, SIGNAL(activated()), this, SLOT(executeShortcut()));
							mapAllSh[dyn->key().toString()] = QString("%1:::::%2").arg(close).arg(command);
							listAllSh[dyn->key().toString()] = dyn;
							currentSH << dyn->key().toString();
						}

					}
				}

				// Disable all the "deleted" shortcuts (they are kept during runtime in case they are reenabled)
				QMapIterator<QString, QString> map(mapAllSh);
				while (map.hasNext()) {
					map.next();
					if(!currentSH.contains(map.key())) {
						listAllSh[map.key()]->setEnabled(false);
					}
				}

			}

		}

	}

}

// A click on a menu item
void MainWindow::menuClicked(QString txt, int close) {

	if(!blockAllFunctions) {

		// If this item was set up so that the menu closes on executing, then do it before executing the actual command (some commands open a QDialog with exec(), that would hold the menu open until after the function call is finished)
		if(close)
			m->animateMenu(m->pos().x()-1,0,true);

		// Also calls this function (same as executeShortcut())
		qDebug() << "MENU:" << txt;
		shortcutAction("__" + txt);

	} else
		qDebug() << "Sorry, blocked...";

}

// This function is called for opening a (new) file - new file opening dialog (BETA!!!!!)
void MainWindow::openFile() {

	if(!blockAllFunctions) {

		thumbs->thread->terminate();
		while(thumbs->thread->isRunning()) {}

		if(thumbs->isHidden())
			thumbs->toggle();
		open->animate();
		opennav->animate();
		qDebug() << "Loading new file.";
		emit blockAll(true);

	} else
		qDebug() << "Sorry, blocked...";

}

// The old open file dialog
void MainWindow::openFileOld() {

	if(!blockAllFunctions) {

		// Stopping a possibly running thread
		thumbs->stopThread();

		qDebug() << "Loading new file.";

		// Open the FileDialog in the dir of last image. If Photo just was started (i.e. no current image), then open in home dir
		QString opendir = QDir::homePath();
		if(currentfile != "")
			opendir = QFileInfo(currentfile).absoluteDir().absolutePath();

		// Get new filename
		QString file = QFileDialog::getOpenFileName(this,tr("Open image file"),opendir,tr("Images") + " (*.bmp *.gif *.tif *.jpg *.jpeg *.jpeg2000 *.png *.pbm *.pgm *.ppm *.xbm *.xpm *.tiff)");

		// If a file was chosen (cancel returns an empoty string)
		if(file.trimmed() != "") {

			whichIsRotated.clear();
			whichIsFlippedVert.clear();
			whichIsFlippedHor.clear();
			manuallyRotated.clear();

			// If thumbnail size was changed -> reload thumbnail settings
			if(thbHeight != set->thumbnail->thumbSizeSlider->value())
				loadSettings();

			// Reset all variables
			currentfile = file;
			thumbs->currentfile = currentfile;
			thumbs->cacheEnabled = set->thumbnail->thumbCache->isChecked();
			thumbs->allimgs.clear();
			thumbs->allPixHov.clear();
			thumbs->allPixImg.clear();
			thumbs->allPixItem.clear();
			thumbs->allPixSel.clear();

			// And load the image and the directory
			thumbs->allImgs();
			qDebug() << "Cache enabled:" << thumbs->cacheEnabled;
			countpos = thumbs->allimgs.indexOf(currentfile);
			counttot = thumbs->counttot;
			drawImg();
			updateCounter();

		} else
			qDebug() << "No new file selected (cancelled?)";

	} else
		qDebug() << "Sorry, blocked...";

}

// A simple print function (still needs to be improved and enhanced a lot)
void MainWindow::print() {

	if(currentfile != "") {

		// USe the standard print dialog provided by Qt
		QPrinter printer(QPrinter::HighResolution);
		QPrintDialog dialog(&printer, this);
		if (dialog.exec()) {
			QPixmap p(currentfile);
			QPainter painter(&printer);
			QRect rect = painter.viewport();
			QSize size = p.size();
			size.scale(rect.size(), Qt::KeepAspectRatio);
			painter.setViewport(rect.x(), rect.y(), size.width(), size.height());
			painter.setWindow(p.rect());
			painter.drawPixmap(0, 0, p);
			painter.end();
		}

	} else

		qDebug() << "Well, you need to load a file first before you can print it...";

}

// The resize event adjusts the position of the dropdown menu and the about widget
void MainWindow::resizeEvent(QResizeEvent *) {

	if(finishedLoading) {

		// The current geometry
		desk = view->geometry();
		m->viewHeight = thumbs->height;

		// Adjust menu position depending on location at top or bottom edge
		if(set->thumbnail->thbPosTop->isChecked()) {
			m->menuRectShown = QRect(desk.width()-580,desk.height()-330,530,330);
			m->menuRectHidden = QRect(m->menuRectShown.x(),desk.height(),m->menuRectShown.width(),m->menuRectShown.height());
			m->posAtBottom = false;
		} else {
			m->menuRectShown = QRect(desk.width()-580,0,530,330);
			m->menuRectHidden = QRect(m->menuRectShown.x(),-m->menuRectShown.height(),m->menuRectShown.width(),m->menuRectShown.height());
			m->posAtBottom = true;
		}

		// And set geometry
		if(m->mShown)
			m->setGeometry(m->menuRectShown);
		else
			m->setGeometry(m->menuRectHidden);


		// Adjust geometry of about widget
		about->rectShown = QRect((desk.width()-900)/2.0,(desk.height()+thumbs->height-550)/2.0,900,550);
		about->rectHidden = QRect(about->rectShown.x(),-about->rectShown.height(),about->rectShown.width(),about->rectShown.height());
		if(about->isShown)
			about->setGeometry(about->rectShown);
		else
			about->setGeometry(about->rectHidden);

//		drawImg();

	}

}

// Rotate the current image
void MainWindow::rotate(int i, bool ignoreblock) {

	if(!blockAllFunctions || ignoreblock) {

		if(currentfile != "") {

			pix->curRotated = true;

			// Save, that file was rotated (i.e. after no change attempt by exif data)
			manuallyRotated[countpos] = true;

			int prevRotation = rotation;

			if(i == 1) {
				rotation -= 90;
				view->rotate(-90);
				qDebug() << "Rotate Left";
			} else if(i == 2) {
				rotation += 90;
				view->rotate(90);
				qDebug() << "Rotate Right";
			} else if(i == 0) {
				view->rotate(-rotation);
				rotation = 0;
				qDebug() << "Reset Rotation";
			}

			// The absolute rotation value is mod 360
			while(rotation > 359)
				rotation -= 360;
			while(rotation < -359)
				rotation += 360;

			// Save rotation state for current file
			whichIsRotated[countpos] = rotation;

			// We adjust the scale factor after rotating (safing us a call of drawImg()), but only if not zoomed
			if(zoomSteps == 0 && prevRotation != rotation) {
				// We reset the scaling in the matrix itself
				view->matrix().setMatrix(1,view->matrix().m12(),1,view->matrix().m22(),view->matrix().dx(),view->matrix().dy());
				// Then we adjust the scaling factor so that the img fits the screen again
				if(pix->pixmap().width() > pix->pixmap().height() && pix->pixmap().width() > view->height()) {
					if(abs(rotation) == 90 || abs(rotation) == 270)
						view->scale(float(view->height())/float(pix->pixmap().width()),float(view->height())/float(pix->pixmap().width()));
					else
						view->scale(float(pix->pixmap().width())/float(view->height()),float(pix->pixmap().width())/float(view->height()));
				}
			}

			// Also rotate the corresponding thumbnail
			thumbs->rotateThumbnail(countpos,i);
		}

	} else
		qDebug() << "Sorry, blocked...";

}

// Set the orientation of the image as stored in exif data
void MainWindow::setExifOrientation(int deg, bool confirmed) {

	if(deg != 0 && whichIsRotated[countpos] == 0 && manuallyRotated.value(countpos) == false) {

		// If confirmed is false (default)
		if(!confirmed) {

			// If user set a tick at "always apply", we apply it
			if(set->exif->exifRotAlw->isChecked()) {
				setExifOrientation(deg,true);
				manuallyRotated[countpos] = true;
			// If user set a tick at "always ask", we ask
			} else if(set->exif->exifRotAsk->isChecked())
				exifRotConf->doRotation(deg);

			// And if the user set a tick at "never apply", we simply do nothing

		} else {

			// A positive degree value means: just rotating , no flipping
			if(deg > 0) {

				qDebug() << "Rotate:" << deg;

				// We rotate the img in 90 degrees steps
				while(deg > 0) {
					rotate(1,true);
					deg -= 90;
				}

			// A negative degree value means: rotate and flip horizontally (values calculated this way)
			} else if(deg < 0) {

				qDebug() << "Flip and rotate:" << abs(deg);

				// Make degree positive
				deg *= -1;

				// We rotate the img in 90 degree steps
				while(deg > 0) {
					rotate(1,true);
					deg -= 90;
					//thumbs->rotateThumbnail(countpos,1);
				}

				// And after we flip it horizontally
				flip(1,true);

			}

			if(zoomSteps == 0)
				zoomAfterRot->start(50);

		}

	// Else if function is called with deg==0, then the file was looked at and user maybe clicked on "Don't do it" - we only ask once, so a "no" is stored for the rest of the session
	} else if(deg == 0 && confirmed)
		manuallyRotated[countpos] = true;

}

// If the user ticked the box "Don't ask again", we store the value in the settings and save the settings to file
void MainWindow::setExifOrientNeverAsk(bool alwaysRotate) {

	if(alwaysRotate)
		set->exif->exifRotAlw->setChecked(true);
	else
		set->exif->exifRotNev->setChecked(true);
	set->saveSettings();

}

// Setup the system tray icon
void MainWindow::setupTrayIcon() {

	// The Tray Icon
	trayIcon = new QSystemTrayIcon(this);
	trayIcon->setIcon(QIcon(":/img/logo.png"));
	trayIcon->setToolTip(tr("Photo - Image Viewer"));

	// A context menu for the tray icon
	trayIconMenu = new QMenu(this);
	trayIconMenu->setStyleSheet("background-color: rgb(67,67,67); color: white; border-radius: 5px;");

	// A new action for the menu
	QAction *trayAcToggle = new QAction(QIcon(":/img/logo.png"),"Hide/Show MainWindow",this);
	trayAcToggle->setObjectName("toggle");
	trayIconMenu->addAction(trayAcToggle);

	// A new action for the menu
	QAction *trayAcLoad = new QAction(QIcon(":/img/open.png"),"Open New Image",this);
	trayAcLoad->setObjectName("open");
	trayIconMenu->addAction(trayAcLoad);

	// A seperator for the menu
	trayIconMenu->addSeparator();

	// A new action for the menu
	QAction *trayAcQuit = new QAction(QIcon(":/img/quit.png"),"Quit Photo",this);
	trayAcQuit->setObjectName("quit");
	trayIconMenu->addAction(trayAcQuit);

	// Set the menu to the tray icon
	trayIcon->setContextMenu(trayIconMenu);

	// Connect signals
	connect(trayAcToggle, SIGNAL(triggered()), this, SLOT(trayAcDo()));
	connect(trayAcLoad, SIGNAL(triggered()), this, SLOT(trayAcDo()));
	connect(trayAcQuit, SIGNAL(triggered()), this, SLOT(trayAcDo()));
	connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(trayAcDo(QSystemTrayIcon::ActivationReason)));

	// If tray icon is actually supposed to be used, show it
	if(set->interface->trayUse->isChecked())
		trayIcon->show();
	else
		trayIcon->hide();

}

// This functions simply takes a string and executes the action set for it
void MainWindow::shortcutAction(QString key, bool mouseAction) {

	if(key == "__stopThb")
		thumbs->stopThread();
	if(key == "__close") {
		skipTrayIcon = true;
		this->close();
	}
	if(key == "__hide")
		this->close();
	if(key == "__settings")
		showSettings();
	if(key == "__next")
		getNext();
	if(key == "__prev")
		getPrev();
	if(key == "__noThumb")
		toggleThumbnails();
	if(key == "__reloadThb")
		thumbs->loadThumbs();
	if(key == "__about")
		about->animate();
	if(key == "__openOld")
		openFile();
	if(key == "__open")
		openFileOld();
	if(key == "__zoomIn") {
		if(mouseAction)
			zoom(1);
		else
			zoom(3);
	}
	if(key == "__zoomOut") {
		if(mouseAction)
			zoom(2);
		else
			zoom(4);
	}
	if(key == "__zoomReset")
		zoom(0);
	if(key == "__zoomActual")
		zoom(0,"zoomToActualSize");
	if(key == "__rotateL")
		rotate(1);
	if(key == "__rotateR")
		rotate(2);
	if(key == "__rotate0")
		rotate(0);
	if(key == "__mirrorH")
		flip(1);
	if(key == "__mirrorV")
		flip(2);
	if(key == "__rename")
		handleFile(1);
	if(key == "__delete")
		handleFile(2);
	if(key == "__copy")
		handleFile(3);
	if(key == "__move")
		handleFile(4);
	if(key == "__hideMeta") {
		if(!exif->isShown) {
			exif->staying = true;
			exif->stay->setText(exif->yesOpen);
			exif->animate(0,0,false,true);
		} else {
			exif->staying = false;
			exif->stay->setText(exif->noOpen);
			exif->animate(0,0,true,false);
		}
	}
	if(key == "__gotoFirstThb")
		thumbs->gotoFirstLast("first");
	if(key == "__gotoLastThb")
		thumbs->gotoFirstLast("last");
	if(key == "__print")
		print();

}

// Show the settings window
void MainWindow::showSettings() {

	if(!blockAllFunctions) {

		qDebug() << "Showing settings window";

		set->setGeometry(100,50,850,650);
		set->shortcuts->doresizing();

		// Move the settings window into the desktop center
		QRect r = set->frameGeometry();
		QDesktopWidget desk;
		r.moveCenter(desk.availableGeometry().center());
		set->move(r.topLeft());

		set->thumbnail->setDatabaseInfo();

		set->exec();

	} else
		qDebug() << "Sorry, blocked...";

}

// Toggles the thumbnails view
void MainWindow::toggleThumbnails(bool alsoRedrawImage) {

	if(!blockAllFunctions) {

		if(currentfile != "") {

			thumbs->toggle();

			// If the function is called manually after an image was loaded, then this is set to false making it possible to manually show thumbnails after they have been automatically hidden for a directory containing only one image
			if(alsoRedrawImage && zoomSteps == 0) {
				toggleThbTimer->setSingleShot(true);
				toggleThbTimer->start(5);
			}

			if(thumbs->isVisible()) {
				m->adjustMenuItems["noThumb"]->setText(tr("Hide Thumbnails"));
				m->adjustMenuItems["noThumb"]->setIcon(":/img/toggle.png");
			} else {
				m->adjustMenuItems["noThumb"]->setText(tr("Show Thumbnails"));
				m->adjustMenuItems["noThumb"]->setIcon(":/img/toggle.png");
			}

		}

	} else
		qDebug() << "Sorry, blocked...";
}

void MainWindow::toggleTimer() {

	drawImg("ignorethumbnails");

}

// Selection of an action in the context menu of the tray icon
void MainWindow::trayAcDo(QSystemTrayIcon::ActivationReason rsn) {

	QAction *s = (QAction *) sender();
	if(s->objectName() == "open") {
		if(this->isHidden())
			this->show();
		openFileOld();
	} else if(s->objectName() == "quit") {
		skipTrayIcon = true;
		this->close();
	} else if(rsn == QSystemTrayIcon::Trigger) {
		if(this->isVisible())
			this->hide();
		else {
			this->showFullScreen();
			qDebug() << currentfile;
			if(currentfile == "--start-in-tray" || currentfile == "") {
				openFileOld();
			}
		}
	}

}

// Update the counter in the top left of the window
void MainWindow::updateCounter() {

	if(!blockAllFunctions) {

		// Only operate on it if image is loaded
		if(currentfile != "") {

			qDebug() << "Updating counter";

			// If position is not hidden, show it
			if(!set->interface->hideCounter->isChecked())
				countLabel->setText(QString("%1/%2").arg(countpos+1).arg(counttot));

			// If filename is not hidden, show it
			if(!set->interface->hideFilename->isChecked()) {
				QString toset = "";
				if(set->thumbnail->thbPosTop->isChecked()) {
					if(!set->interface->hideCounter->isChecked())
						toset += " -";
					toset = currentfile + toset;
				} else {
					if(!set->interface->hideCounter->isChecked())
						toset += "- ";
					toset += currentfile;
				}
				filenameLabel->setText(toset);
			}

			// Adjust the label's size
			countLabel->adjustSize();
			filenameLabel->adjustSize();

			// We hide the labels completely if they don't display any text anyway
			if(set->interface->hideCounter->isChecked())
				countLabel->hide();
			else
				countLabel->show();
			if(set->interface->hideFilename->isChecked())
				filenameLabel->hide();
			else
				filenameLabel->show();

			// Hide these two labels when photo run as --start-in-tray as long as no img is loaded
			if(currentfile == "--start-in-tray") {
				filenameLabel->hide();
				countLabel->hide();
			}


		}

	} else
		qDebug() << "Sorry, blocked...";

}

// Zoom the image
void MainWindow::zoom(int z, QString special) {

	if(!blockAllFunctions) {

		// Only do it if a file is loaded
		if(currentfile != "") {

			if(special == "zoomToActualSize") {

				view->setTransformationAnchor(view->AnchorViewCenter);

				qDebug() << "Showing actual image size";

				verScaled = 1.0;
				horScaled = 1.0;
				zoomSteps = 0;

				// First draw the full image
				drawImg("zoomactual");

				// Reset all the scaling factors
				verScaled = 1.0;
				horScaled = 1.0;
				zoomSteps = 0;

				// And save the scaling position
				float temp = 0.0;

				if(float(pix->pixmap().height())/float(pix->pixmap().width()) < float(desk.height())/float(desk.width()))
					temp = ((pix->pixmap().height()-desk.height()-thbHeight-scrollbarheight))/(0.15*(desk.height()+thbHeight+scrollbarheight));
				else
					temp = (pix->pixmap().width()-desk.width())/(0.15*(desk.width()));
				if(temp >= 0) {
					horScaled = temp;
					verScaled = temp;
				}
				zoomSteps = (horScaled-1.0)/0.15;

			} else  {

				// If the zoom was called from the mouse wheel and ctrl being pressed: Center on mouse
				if(z < 3)
					view->setTransformationAnchor(view->AnchorUnderMouse);
				// If zoom was called per shortcut or from menu: Center on center of screen
				else {
					view->setTransformationAnchor(view->AnchorViewCenter);
					z -= 2;
				}

				// Get the current zoom position
				int temp = zoomSteps;
				float currentZoom = 1.0;
				while(temp < 0) {
					currentZoom = currentZoom/(currentZoom+0.15);
					++temp;
				}
				while(temp > 0) {
					currentZoom = (currentZoom+0.15)/currentZoom;
					--temp;
				}

				// Scale view by this integer
				float doZoom = 1.0;

				// If the image currently is still in the normal size (fitting the screen), then draw either the full image scaled down for zooming in or the normal image for zooming out
				if(zoomSteps == 0 && z != 0) {
					if(z == 1)
						drawImg("zoomin");
					else if(z == 2)
						drawImg("zoomout");
				}

				// Zooming in (limited by zoom factor 12)
				if(z == 1 && verScaled < 12) {
					verScaled += 0.15;
					horScaled += 0.15;
					doZoom = (currentZoom+0.15)/currentZoom;
					zoomSteps += 1;
					// If the user returns through the zooming in step to the normal position, we redraw the image
					if(zoomSteps == 0)
						zoom(0);
					else if(verScaled > 0)
						view->scale(doZoom,doZoom);
				// Zooming out (limited)
				} else if(z == 2 && verScaled > 0.10) {
					verScaled -= 0.15;
					horScaled -= 0.15;
					doZoom = currentZoom/(currentZoom+0.15);
					zoomSteps -= 1;
					if(zoomSteps == 0)
						zoom(0);
					else
						view->scale(doZoom,doZoom);
				// Reset the zooming position
				} else if(z == 0) {
					verScaled = 1.0;
					horScaled = 1.0;
					zoomSteps = 0;
					view->resetMatrix();
					drawImg("zoomreset");
				}

			}

		} else
			qDebug() << "Can't zoom, no file loaded yet";

	} else
		qDebug() << "Sorry, blocked...";



}

MainWindow::~MainWindow() { }




/*####################################################################################
  ####################################################################################
  ####################################################################################
  ####################################################################################
  ####################################################################################*/


// A custom label for enabling a right click on them, and adding a clicked() event
Label::Label(QWidget *parent, QString objName) : QLabel(parent) {

	this->setObjectName(objName);

	c = new QMenu;
	dohide = new QAction(tr("Hide"),c);
	dohide->setObjectName(objName);
	c->addAction(dohide);

}

// The label on the top right (the little "x") for closing the app
void Label::mouseReleaseEvent(QMouseEvent *e) {

	if(e->button() == Qt::LeftButton)
		emit clicked();

}

// Showing the context menu
void Label::contextMenuEvent(QContextMenuEvent*) {
	c->popup(QCursor::pos());
}

Label::~Label() { }
