#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "image.h"
#include "thumbnail.h"
#include "settings.h"
#include "menu.h"
#include "filehandling.h"
#include "exifdata.h"
#include "graphicsitem.h"
#include "about.h"
#include "exifrotconfirm.h"
#include "open.h"
#include "opennav.h"

#include <QMainWindow>
#include <QApplication>
#include <QtConcurrentRun>
#include <QFutureWatcher>
#include <QProcess>
#include <QFileDialog>
#include <QSignalMapper>
#include <QScrollBar>
#include <QMapIterator>
#include <QMovie>
#include <QGraphicsProxyWidget>
#include <QMenu>

#include <QPrinter>
#include <QPrintDialog>

#include <QSystemTrayIcon>

// A custom label (the "x" top right corner) for closing Photo
class Label : public QLabel {
	Q_OBJECT

public:
	Label(QWidget *parent = 0, QString objName = "");
	~Label();
	QMenu *c;
	QAction *dohide;
	QString version;

protected:
	void mouseReleaseEvent(QMouseEvent *e);
	void contextMenuEvent(QContextMenuEvent *ev);
signals:
	void clicked();
};

// The main window
class MainWindow : public QMainWindow {

	Q_OBJECT

public:

	explicit MainWindow(QWidget *parent = 0);
	~MainWindow();

	// The main widgets
	Image *view;
	Settings *set;
	Thumbnails *thumbs;
	QGraphicsScene scene;

	// This is responsible for showing the big image
	GraphicsItem *pix;

	// That's the currently and previously viewed picture
	QString currentfile;
	QString previousfile;

	// The label at the top left, showing the current position in the directory
	Label *countLabel;
	int counttot;
	int countpos;

	Label *filenameLabel;

	// A Label for closing photo quick and easy
	Label *doClose;

	// This integer stores the size of the thumbnails and the thumbnails view
	int thbHeight;
	int scrollbarheight;

	// This shortcuts are already set up -> no need to change (after closing settings window)
	QStringList alreadySetupShortcuts;

	// These two QMaps hold all the info regarding shortcuts
	QMap<QString,QString> mapAllSh;
	QMap<QString,QShortcut*> listAllSh;

	// This string holds the current version
	QString version;

	// Some variables for the zooming story
	float absoluteZoomFactor;
	float horScaled;
	float verScaled;
	int zoomSteps;

	// Menu
	Menu *m;

	// Some variables for the rotating
	int rotation;
	QMap<int,int> whichIsRotated;
	QMap<int, bool> whichIsFlippedVert;
	QMap<int, bool> whichIsFlippedHor;
	QMap<int,bool> manuallyRotated;

	// File Handling Widget (renaming/deleting)
	HandleFile *handle;

	// Boolean to block all functions
	bool blockAllFunctions;

	// This DesktopWidget is used to get the screen dimensions
	QRect desk;

	// A widget for displaying exif information
	Exif *exif;

	// A widget for displaying the about information
	About *about;

	// Widget for confirming rotation
	ExifRotConf *exifRotConf;

	// Adjust zooming
	QTimer *zoomAfterRot;

	// Toggle THB Timer -> redraw image
	QTimer *toggleThbTimer;

	// A new open file dialog (BETA!!!!)
	Open *open;
	OpenNav *opennav;

	// Setup and handling of tray icon (optional), used for faster access of photo
	void setupTrayIcon();
	QSystemTrayIcon *trayIcon;
	QMenu *trayIconMenu;
	bool skipTrayIcon;

	// This timer ensures that only one instance of photo can run at one time, and makes photo remote controllable
	QTimer *globalTimer;

	bool finishedLoading;

public slots:
	// After the settings have been closed, some variables need to be adjusted
	void afterSettingsClosed();

	// Set all booleans also in subclasses for blocking Photo
	void blockAll(bool block = false);

	// Calls drawImg() on startup whenever the scene was correctly setup
	void doDrawImgStartUp();

	// Handles the hide command issued from the context menu of the labels of the class Label
	void doHideLabel();

	// This funtion shows the image (path storred in currentfile)in big
	void drawImg(QString dothis = "normal");

	// Execute a shortcut (actually execute command)
	void executeShortcut();

	// Flip the image vertically/horizontally
	void flip(int, bool ignoreblock = false);

	// When the settings window is closed, make sure, that the mainwindow regains the focus (that must not be, if the user switched between windows while the settings window was opened)
	void focusThisWindow();

	// Get the next/previous image in directory
	void getNext();
	void getPrev();

	// A click on the scene -> quit Photo if outside image and corresponsing option is set
	void getSceneClick(QPoint);

	// A global timer ensures only one instance of photo can run at one time, and makes photo remotely controllable
	void globalTimerDO();

	// File Handling functions
	void handleFile(int todo);
	void doRename(QString);
	void doDelete();

	// Loads all shortcuts
	void loadShortcuts();

	// After click on a thumbnail, the associated image is loaded
	void loadNewFile(QString file);
	void loadNewImg(QString, int);

	// Load the thumbnail settings (the thumbnail size can be adjusted by the user)
	void loadSettings();

	// A click on a menu item
	void menuClicked(QString, int);

	// Opens a (new) file
	void openFile();
	void openFileOld();

	// Provisional print function (still very simple!!)
	void print();

	// Rotating the image
	void rotate(int, bool ignoreblock = false);

	// Function for a rotation stored in exif data
	void setExifOrientation(int deg, bool confirmed = false);
	void setExifOrientNeverAsk(bool alwaysRotate);

	// Executes a shortcut
	void shortcutAction(QString, bool mouseAction = false);

	// Show the settings window
	void showSettings();

	// Reset zoom with timer
	void timerResetZoom() {
		zoom(0);
	}

	// Toggle the thumbnails
	void toggleThumbnails(bool alsoRedrawImage = true);
	void toggleTimer();

	// Called by actions selected from tray menu
	void trayAcDo(QSystemTrayIcon::ActivationReason rsn = QSystemTrayIcon::Trigger);

	// After an image is drawn, the counter top left is updated
	void updateCounter();

	// Zooming the image
	void zoom(int, QString special = "nothing");

protected:
	void closeEvent(QCloseEvent *);
	void resizeEvent(QResizeEvent *);

};

#endif // MAINWINDOW_H
