#include "open.h"

Open::Open(QWidget *parent) : QWidget(parent) {

	// Style the widget
	this->setStyleSheet("background-color: rgba(0,0,0,200);");

	// The different QRects for its shapes
	rectHidden = QRect(this->x(),this->y(),this->width(),-this->height());
	aniStart = QRect(this->width()/2.0,this->height()/2.0,1,1);
	rectShown = QRect(this->x(),this->y(),this->width(),this->height());

	// The GraphicsView that'll contain all the items
	view = new QGraphicsView;
	view->setScene(&scene);
	view->setMouseTracking(true);
	view->setAlignment(Qt::AlignTop | Qt::AlignLeft);
	view->setStyleSheet("background: transparent");

	// The default dir is the home directory
	dir = QDir::homePath();

	// That's the current position in the history - there is none
	currentPos = -1;

	// The current dir is shown in "breadcrumb" style - make them clickable in a future release...
	QString dirTmp = dir;
	pos = new QLabel(dirTmp.replace("/"," > "));
	pos->setStyleSheet("color: white; background: transparent");

	// A label displaying a warning, that this is still beta
	beta = new QLabel("STILL BETA - USE AT OWN RISK");
	beta->setStyleSheet("color: red; background: transparent; font-weight: bold");

	// A button to close the open dialog
	close = new QPushButton(tr("CLOSE"));
	close->setStyleSheet("color: white; font-weight: bold; padding: 5px; border-radius: 10px;");

	// A layout for the labels and button
	QHBoxLayout *posLay = new QHBoxLayout;
	posLay->addWidget(pos);
	posLay->addStretch();
	posLay->addWidget(beta);
	posLay->addSpacing(25);
	posLay->addWidget(close);

	// The layout of the graphicsview
	QVBoxLayout *right = new QVBoxLayout;
	right->addWidget(view);
	right->addLayout(posLay);
	right->setMargin(0);

	// A TreeView showing a directory tree
	tree = new QTreeView;
	tree->setMaximumWidth(350);
	tree->setMinimumWidth(350);
	model = new QFileSystemModel;
	tree->setModel(model);
	model->setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
	model->setRootPath(QDir::homePath());
	model->sort(0);
	// We only show the first column, the directory name
	for(int i = 1; i < model->columnCount(); ++i)
		tree->hideColumn(i);
	tree->hideColumn(1);
	tree->setStyleSheet("color: white");
	tree->setRootIndex(model->index(QDir::homePath()));

	// That's the main layout of the open widget
	QHBoxLayout *main = new QHBoxLayout;
	main->addWidget(tree);
	main->addLayout(right);
	this->setLayout(main);

	// Needed for hovering/etc. effects
	this->setMouseTracking(true);

	// This thread creates the thumbnails and updates the view
	openThread = new ThumbThread;
	connect(openThread, SIGNAL(updateThb(QImage,QString,int)), this, SLOT(updateThumb(QImage,QString,int)));

	// Initially this widget is hidden
	this->setGeometry(rectHidden);
	oShown = false;

	// That's the animation object
	ani = new QPropertyAnimation(this,"geometry");

	// Connect a couple signals
	connect(ani, SIGNAL(finished()), this, SLOT(loadDir()));
	connect(model, SIGNAL(directoryLoaded(QString)), this, SLOT(clickOnTree(QString)));

}

// Adjust the QRects
void Open::resizeMe(int width, int height) {

	rectHidden = QRect(this->x(),this->x(),width,-height);
	rectShown = QRect(0,0,width, height);
	aniStart = QRect(50,50,1,1);

	// Update size immediately if widget is shown
	if(oShown && ani->state() == QPropertyAnimation::Stopped)
		this->setGeometry(rectShown);

}

// Animate open widget
void Open::animate() {

	// Tell navigation class, that the open dialog is being animated
	emit animating();

	// Open widget
	if(ani->state() == QPropertyAnimation::Stopped && !oShown) {
		ani->setDuration(600);
		qDebug() << "Opening Widget";
		oShown = true;

		// Hide all the sub-widgets while the main widget is being zoomed in
		view->hide();
		pos->hide();
		beta->hide();
		close->hide();
		tree->hide();

		ani->setStartValue(aniStart);
		ani->setEndValue(rectShown);
		ani->setEasingCurve(QEasingCurve::InBack);
		ani->start();
	// Close widget
	} else if(ani->state() == QPropertyAnimation::Stopped && oShown) {
		ani->setDuration(300);
		qDebug() << "Closing Widget";
		oShown = false;

		// Hide all the sub-widgets while the main widget is being zoomed out
		view->hide();
		pos->hide();
		beta->hide();
		close->hide();
		tree->hide();

		ani->setStartValue(rectShown);
		ani->setEndValue(aniStart);
		ani->setEasingCurve(QEasingCurve::OutBack);
		ani->start();
	}

}

// Go back in the history
void Open::goBack() {

	if(currentPos > 0) {
		--currentPos;
		dir = history.at(currentPos);
		loadDir(false);

	}

}

// Go forward in the history
void Open::goForward() {

	if(currentPos+1 < history.length()) {
		++currentPos;
		dir = history.at(currentPos);
		loadDir(false);

	}

}

// Update a thumbnail
void Open::updateThumb(QImage p, QString f, int pos) {

	// Default size
	int width = 120;
	int height = 90;

	// The thumbnail image
	QPixmap pix(100,100);
	pix.fill(Qt::transparent);
	QPixmap thb = QPixmap::fromImage(p);
	QPainter pixp(&pix);

	// The position of the first item
	int x = 0;
	int y = 0;
	int w = 100;
	int h = 100;

	// We position the thb according to it's size
	if(thb.width() > thb.height()) {
		h = int((100.0/float(width))*pix.height());
		y = (100-h)/2;
	} else {
		w = int((100.0/float(height))*pix.width());
		x = (100-w)/2;
	}

	// Draw the pixmap
	pixp.drawPixmap(x,y,w,h,thb);
	pixp.end();

	// The normal image
	QPixmap imgNorm(width,height);
	imgNorm.fill(Qt::transparent);
	QPainter paint(&imgNorm);
	paint.setCompositionMode(QPainter::CompositionMode_SourceOver);
	paint.drawPixmap(29,14,62,62,pix);
	QTextDocument txt;
	txt.setHtml("<center><div style=\"text-align: center; font-size: 8pt; font-wight: bold; color: white\">" + QFileInfo(f).fileName() + "</div></center>");
	txt.setTextWidth(120);
	paint.translate(3,height*2/3.0);
	txt.drawContents(&paint);
	paint.end();

	// The hover image
	QPixmap imgHov(width,height);
	imgHov.fill(Qt::transparent);
	QPainter paintSel(&imgHov);
	paintSel.setCompositionMode(QPainter::CompositionMode_SourceOver);
	paintSel.drawPixmap(30,15,60,60,pix);
	QTextDocument txtSel;
	txtSel.setHtml("<center><div style=\"text-align: center; font-size: 8pt; font-wight: bold; color: white\">" + QFileInfo(f).fileName() + "</div></center>");
	txtSel.setTextWidth(120);
	paintSel.translate(3,height*2/3.0);
	txtSel.drawContents(&paintSel);
	paintSel.end();

	allPixmaps.at(pos)->setPixmap(imgNorm,imgHov);

}

// Load a new directory
void Open::loadDir(bool updateHistory) {

	openThread->terminate();

	// Update TreeView
	tree->selectionModel()->clearSelection();
	tree->selectionModel()->select(model->index(dir), QItemSelectionModel::Select);
	tree->expand(model->index(dir));

	// Show all the widgets
	view->show();
	pos->show();
	beta->show();
	tree->show();
	close->show();

	// Remove all items from the view
	allPixmaps.clear();

	// Update the history
	if(updateHistory) {
		if(currentPos+1 < history.length()) {
			int len = history.length();
			for(int i = currentPos+1; i < len; ++i)
				history.removeLast();
		}
		history.append(dir);
		++currentPos;
	}

	// Dis-/Enable navigation widget

	if(currentPos+1 == history.length())
		emit enableForward(false);
	else
		emit enableForward(true);

	if(currentPos == 0)
		emit enableBack(false);
	else
		emit enableBack(true);

	if(dir == QDir::rootPath())
		emit enableUp(false);
	else
		emit enableUp(true);

	// Update the current position
	QString dirTmp = dir;
	pos->setText(dirTmp.replace("/"," > "));

	// Clear the scene
	scene.clear();

	// How many items in a row
	int c = view->width()/120;

	// Access the directory
	QDir *d = new QDir(dir);

	// First only get the directories in the directory
	d->setFilter(QDir::Dirs | QDir::NoDotAndDotDot);

	// Get the information about all the directories
	QFileInfoList foldersInfo = d->entryInfoList();
	QStringList foldersName = d->entryList();

	// Access the directory again
	QDir *f = new QDir(dir);

	// These are the images known by photo
	QStringList flt;
	flt << "*.bmp" << "*.gif" << "*.tif" << "*.jpg" << "*.jpeg" << "*.jpeg2000" << "*.png" << "*.pbm" << "*.pgm" << "*.ppm" << "*.xbm" << "*.xpm" << "*.tiff";
	f->setNameFilters(flt);

	// Store a QFileInfoList and a QStringList with the filenames
	imgInfo = f->entryInfoList(QDir::NoFilter,QDir::IgnoreCase);
	imgName = f->entryList();

	qDebug() << "Loading dir";


	///////////////////////////////
	//// LOADING FOLDERS FIRST ////
	///////////////////////////////


	// We start in the top left corner
	int row = 1;
	int column = 0;

	// Adjust scene rect
	scene.setSceneRect(scene.itemsBoundingRect());

	// Loop through allo the folders
	for(int i = 0; i < foldersName.size(); ++i) {

		// The current item name
		QString str = foldersName.at(i);

		// Current size
		int width = 120;
		int height = 90;

		// The normal folder
		QPixmap folderNorm(width,height);
		folderNorm.fill(Qt::transparent);
		QPainter paint(&folderNorm);
		paint.setCompositionMode(QPainter::CompositionMode_SourceOver);
		QIcon f(QIcon::fromTheme("folder"));
		paint.drawPixmap(29,14,62,62,f.pixmap(80,80));
		QTextDocument txt;
		txt.setHtml("<center><div style=\"text-align: center; font-size: 8pt; font-wight: bold; color: white\">" + str + "</div></center>");
		txt.setTextWidth(120);
		paint.translate(3,height*2/3.0);
		txt.drawContents(&paint);
		paint.end();

		// The hovered folder
		QPixmap folderHov(width,height);
		folderHov.fill(Qt::transparent);
		QPainter paintSel(&folderHov);
		paintSel.setCompositionMode(QPainter::CompositionMode_SourceOver);
		QIcon fSel(QIcon::fromTheme("folder-cyan"));
		paintSel.drawPixmap(30,15,60,60,fSel.pixmap(80,80));
		QTextDocument txtSel;
		txtSel.setHtml("<center><div style=\"text-align: center; font-size: 8pt; font-wight: bold; color: white\">" + str + "</div></center>");
		txtSel.setTextWidth(120);
		paintSel.translate(3,height*2/3.0);
		txtSel.drawContents(&paintSel);
		paintSel.end();

		// Create Pixmap
		Pixmap *pixFold = new Pixmap;
		pixFold->setPixmap(folderNorm,folderHov);
		pixFold->itemType = "folder";
		pixFold->path = foldersInfo.at(i).absoluteFilePath();

		connect(pixFold, SIGNAL(clicked(QString,QString)), this, SLOT(gotClick(QString,QString)));

		pixFold->setPos(column*120,row*90);

		// Get pos of next item
		++column;
		if(column == c) {
			column = 0;
			row += 1;
		}

		// Add item to GraphicsView
		scene.addItem(pixFold);

	}

	///////////////////////////////
	//// LOADING IMAGES SECOND ////
	///////////////////////////////

	// Loop through all images
	for(int i = 0; i < imgName.size(); ++i) {

		// The current item name
		QString str = imgName.at(i);

		// The current item size
		int width = 120;
		int height = 90;

		// The normal image pixmap
		QPixmap imgNorm(width,height);
		imgNorm.fill(Qt::transparent);
		QPainter paint(&imgNorm);
		paint.setCompositionMode(QPainter::CompositionMode_SourceOver);
		QIcon f(QIcon::fromTheme("image-x-generic"));
		paint.drawPixmap(29,14,62,62,f.pixmap(80,80));
		QTextDocument txt;
		txt.setHtml("<center><div style=\"text-align: center; font-size: 8pt; font-wight: bold; color: white\">" + str + "</div></center>");
		txt.setTextWidth(120);
		paint.translate(3,height*2/3.0);
		txt.drawContents(&paint);
		paint.end();

		// The hovered image pixmap
		QPixmap imgHov(width,height);
		imgHov.fill(Qt::transparent);
		QPainter paintSel(&imgHov);
		paintSel.setCompositionMode(QPainter::CompositionMode_SourceOver);
		QIcon fSel(QIcon::fromTheme("image-x-generic"));
		paintSel.drawPixmap(30,15,60,60,fSel.pixmap(80,80));
		QTextDocument txtSel;
		txtSel.setHtml("<center><div style=\"text-align: center; font-size: 8pt; font-wight: bold; color: white\">" + str + "</div></center>");
		txtSel.setTextWidth(120);
		paintSel.translate(3,height*2/3.0);
		txtSel.drawContents(&paintSel);
		paintSel.end();

		// The tooltip of each item
		QString tooltip = "<div style=\"background-color: rgba(0,0,0,230); padding: 15px; border-radius: 3px;\"><b><span style=\"font-size: 13pt\">" + str + "</span></b><br><br>";
		tooltip += QString("<b>" + tr("Size") + "</b>:  %1 MB<br>").arg(qRound(10*(float(imgInfo.at(i).size())/1024.0/1024.0))/10.0);
		tooltip += QString("<b>" + tr("Width") + ":</b> %1 px<br>").arg(QImageReader(imgInfo.at(i).filePath()).size().width());
		tooltip += QString("<b>" + tr("Height") + ":</b> %1 px<br>").arg(QImageReader(imgInfo.at(i).filePath()).size().height());
		tooltip += QString("<b>" + tr("Created") + ":</b> %1<br>").arg(imgInfo.at(i).created().toString());
		tooltip += QString("<b>" + tr("Last Changed") + ":</b> %1</div>").arg(imgInfo.at(i).lastModified().toString());

		// Create the Pixmap
		Pixmap *pixImg = new Pixmap;
		pixImg->setPixmap(imgNorm,imgHov);
		pixImg->itemType = "image";
		pixImg->path = imgInfo.at(i).absoluteFilePath();
		pixImg->setToolTip(tooltip);
		allPixmaps.append(pixImg);

		connect(pixImg, SIGNAL(clicked(QString,QString)), this, SLOT(gotClick(QString,QString)));

		pixImg->setPos(column*120,row*90);

		// Update the position for the next item
		++column;
		if(column == c) {
			column = 0;
			row += 1;
		}

		// Add item to Scene
		scene.addItem(pixImg);

	}

	// Adjust the scene rect
	scene.setSceneRect(scene.itemsBoundingRect());

	// Start the thread
	openThread->counttot = imgInfo.length();
	openThread->allimgs.clear();
	openThread->allimgs.append(imgInfo);
	openThread->height = 80;
	openThread->cacheEnabled = true;
	openThread->breakme = false;
	openThread->dbName = "open";
	if(imgInfo.length())
		openThread->start();

}

// A dir was selected in the TreeView
void Open::clickOnTree(QString d) {

	dir = d;
	loadDir();

}

// A click on an item
void Open::gotClick(QString itemType, QString path) {

	// If a folder was clicked - load folder
	if(itemType == "folder") {

		dir = path;
		loadDir();

	// If image was clicked - load image
	} else if(itemType == "image") {

		openThread->terminate();

		emit loadNewDir(path);
		dir = QFileInfo(path).absolutePath();
		animate();

	}

}

// Go a level up
void Open::goUp() {

	QDir u = QDir(dir);
	u.cdUp();
	dir = u.absolutePath();
	loadDir();

}

// Go home
void Open::goHome() {

	dir = QDir::homePath();
	loadDir();

}



Open::~Open() { }

// Enabling styling of widget
void Open::paintEvent(QPaintEvent *) {
	QStyleOption o;
	o.initFrom(this);
	QPainter p(this);
	style()->drawPrimitive(QStyle::PE_Widget, &o, &p, this);
}
