#include "settings.h"

Settings::Settings(QWidget *parent) : QDialog(parent) {

	// Set window title
	this->setWindowTitle(tr("Photo Settings"));

	// the main central layout
	QVBoxLayout *centralLayout = new QVBoxLayout;
	this->setLayout(centralLayout);

	// The tab widget (for "Shortcuts" and "General Settings" Tabs)
	tabs = new QTabWidget;
	centralLayout->addWidget(tabs);
	tabs->setTabPosition(QTabWidget::North);

	// Tab 1: Shortcuts
	shortcuts = new Shortcuts(this);
	tabs->addTab(shortcuts,tr("&Shortcuts"));

	// Tab 2: Interface Settings
	interface = new Interface(this);
	tabs->addTab(interface,tr("&Interface Settings"));

	// Tab 3: Thumbnail Settings
	thumbnail = new Thumbnail(this);
	tabs->addTab(thumbnail,tr("&Thumbnail Settings"));

	// Tab 4: Exif Settings
	exif = new TabExif(this);
	tabs->addTab(exif,tr("&Exif Information"));

	// Tab 5: Language Settings
	lang = new Lang(this);
	tabs->addTab(lang,tr("&Language"));

	// The two buttons on the bottom of the setting window (always present)
	QPushButton *cancel = new QPushButton(tr("Cancel (Escape)"));
	cancel->setShortcut(QKeySequence("Escape"));
	QPushButton *save = new QPushButton(tr("Save and close (Ctrl+S)"));
	save->setShortcut(QKeySequence("Ctrl+S"));
	QHBoxLayout *buttBot = new QHBoxLayout;
	buttBot->addStretch();
	buttBot->addWidget(cancel);
	buttBot->addWidget(save);
	buttBot->addStretch();
	centralLayout->addLayout(buttBot);

	// Cancel and reverse changes
	connect(cancel, SIGNAL(clicked()), this, SLOT(close()));
	connect(cancel, SIGNAL(clicked()), shortcuts, SLOT(loadShortcuts()));
	connect(cancel, SIGNAL(clicked()), this, SLOT(loadSettings()));

	// Save changes and close window
	connect(save, SIGNAL(clicked()), shortcuts, SLOT(saveData()));
	connect(save, SIGNAL(clicked()), this, SLOT(saveSettings()));
	connect(save, SIGNAL(clicked()), this, SLOT(close()));

	loadSettings();

}

// Focus the first tab on close
void Settings::closeEvent(QCloseEvent *) {
	tabs->setCurrentIndex(0);
}

// Load the current settings
void Settings::loadSettings() {

	QFile file(QDir::homePath() + "/.photo/settings");

	if(!file.open(QIODevice::ReadOnly))

		QMessageBox::critical(0,tr("Couldn't open settings file"),tr("<b>ERROR!</b><br><br>Couldn't open the settings file. Please ensure, that you have read and write access to your home directory!"));

	else {

		QTextStream in(&file);
		QString all = in.readAll();

		// English is selected as default language
		lang->language->item(0)->setSelected(true);

		// Check if another language is to be selected
		for(int i = 0; i < lang->language->count(); ++i) {
			if(all.contains("Language=" + lang->language->item(i)->data(Qt::UserRole+1).toString()))
				lang->language->item(i)->setSelected(true);
		}

		if(all.contains("Composite=1"))
			interface->composite->setChecked(true);
		else
			interface->composite->setChecked(false);

		if(all.contains("TrayIcon=0"))
			interface->trayUse->setChecked(false);
		else
			interface->trayUse->setChecked(true);

		if(all.contains("Transition="))
			interface->transition->setValue(all.split("Transition=").at(1).split("\n").at(0).toInt());
		else
			interface->transition->setValue(0);

		if(all.contains("LoopThroghFolder=0"))
			interface->loop->setChecked(false);
		else
			interface->loop->setChecked(true);

		if(all.contains("MenuSensitivity="))
			interface->menu->setValue(all.split("MenuSensitivity=").at(1).split("\n").at(0).toInt());
		else
			interface->menu->setValue(4);

		if(all.contains("CloseOnGrey=1"))
			interface->grey->setChecked(true);
		else
			interface->grey->setChecked(false);

		if(all.contains("HideCounter=1"))
			interface->hideCounter->setChecked(true);
		else
			interface->hideCounter->setChecked(false);

		if(all.contains("HideFilename=1"))
			interface->hideFilename->setChecked(true);
		else
			interface->hideFilename->setChecked(false);

		if(all.contains("HideX=1"))
			interface->hideX->setChecked(true);
		else
			interface->hideX->setChecked(false);

		if(all.contains("ThumbnailSize="))
			thumbnail->thumbSizeSlider->setValue(all.split("ThumbnailSize=").at(1).split("\n").at(0).toInt());
		else
			thumbnail->thumbSizeSlider->setValue(80);

		if(all.contains("ThumbnailPosition=Top"))
			thumbnail->thbPosTop->setChecked(true);
		else
			thumbnail->thbPosBot->setChecked(true);

		if(all.contains("ThumbnailCache=0")) {
			thumbnail->thumbCache->setChecked(false);
			thumbnail->cacheDatabase->setEnabled(false);
			thumbnail->cacheFile->setEnabled(false);
			thumbnail->emptyDatabase->setEnabled(false);
			thumbnail->cleanDatabase->setEnabled(false);
		} else {
			thumbnail->thumbCache->setChecked(true);
			thumbnail->cacheDatabase->setEnabled(true);
			thumbnail->cacheFile->setEnabled(true);
			thumbnail->emptyDatabase->setEnabled(true);
			thumbnail->cleanDatabase->setEnabled(true);
		}

		if(all.contains("ThbCacheFile=1")) {
			thumbnail->cacheFile->setChecked(true);
			thumbnail->cacheDatabase->setChecked(false);
		} else {
			thumbnail->cacheDatabase->setChecked(true);
			thumbnail->cacheFile->setChecked(false);
		}

		if(all.contains("ExifFilename=0"))
			exif->allCheckBoxes["filename"]->setChecked(false);
		else
			exif->allCheckBoxes["filename"]->setChecked(true);

		if(all.contains("ExifFiletype=0"))
			exif->allCheckBoxes["filetype"]->setChecked(false);
		else
			exif->allCheckBoxes["filetype"]->setChecked(true);

		if(all.contains("ExifFilesize=0"))
			exif->allCheckBoxes["filesize"]->setChecked(false);
		else
			exif->allCheckBoxes["filesize"]->setChecked(true);

		if(all.contains("ExifDimensions=0"))
			exif->allCheckBoxes["dimensions"]->setChecked(false);
		else
			exif->allCheckBoxes["dimensions"]->setChecked(true);

		if(all.contains("ExifMake=1"))
			exif->allCheckBoxes["make"]->setChecked(true);
		else
			exif->allCheckBoxes["make"]->setChecked(false);

		if(all.contains("ExifModel=1"))
			exif->allCheckBoxes["model"]->setChecked(true);
		else
			exif->allCheckBoxes["model"]->setChecked(false);

		if(all.contains("ExifSoftware=1"))
			exif->allCheckBoxes["software"]->setChecked(true);
		else
			exif->allCheckBoxes["software"]->setChecked(false);

		if(all.contains("ExifPhotoTaken=0"))
			exif->allCheckBoxes["phototaken"]->setChecked(false);
		else
			exif->allCheckBoxes["phototaken"]->setChecked(true);

		if(all.contains("ExifExposureTime=0"))
			exif->allCheckBoxes["exposuretime"]->setChecked(false);
		else
			exif->allCheckBoxes["exposuretime"]->setChecked(true);

		if(all.contains("ExifFlash=0"))
			exif->allCheckBoxes["flash"]->setChecked(false);
		else
			exif->allCheckBoxes["flash"]->setChecked(true);

		if(all.contains("ExifIso=0"))
			exif->allCheckBoxes["iso"]->setChecked(false);
		else
			exif->allCheckBoxes["iso"]->setChecked(true);

		if(all.contains("ExifSceneType=1"))
			exif->allCheckBoxes["scenetype"]->setChecked(true);
		else
			exif->allCheckBoxes["scenetype"]->setChecked(false);

		if(all.contains("ExifFLength=1"))
			exif->allCheckBoxes["flength"]->setChecked(true);
		else
			exif->allCheckBoxes["flength"]->setChecked(false);

		if(all.contains("ExifFNumber=1"))
			exif->allCheckBoxes["fnumber"]->setChecked(true);
		else
			exif->allCheckBoxes["fnumber"]->setChecked(false);

		if(all.contains("ExifLightSource=1"))
			exif->allCheckBoxes["lightsource"]->setChecked(true);
		else
			exif->allCheckBoxes["lightsource"]->setChecked(false);

		if(all.contains("ExifGps=1"))
			exif->allCheckBoxes["gps"]->setChecked(true);
		else
			exif->allCheckBoxes["gps"]->setChecked(false);

		if(all.contains("ExifRotation=Always"))
			exif->exifRotAlw->setChecked(true);
		else if(all.contains("ExifRotation=Never"))
			exif->exifRotNev->setChecked(true);
		else
			exif->exifRotAsk->setChecked(true);

		if(all.contains("ExifGPSMapService=bing.com/maps"))
			exif->exifGPSBrowser->item(1)->setSelected(true);
		else
			exif->exifGPSBrowser->item(0)->setSelected(true);

		file.close();

	}

}

// Save all settings to file
void Settings::saveSettings() {

	qDebug() << "Saving settings";

	QFile file(QDir::homePath() + "/.photo/settings");

	if(!file.open(QIODevice::ReadWrite))

		QMessageBox::critical(0,tr("Couldn't open settings file"),tr("<b>ERROR!</b><br><br>Couldn't open the settings file. Please ensure, that you have read and write access to your home directory!"));

	else {

		file.close();
		file.remove();
		file.open(QIODevice::ReadWrite);

		QTextStream out(&file);

		QString cont = "Version=" + version + "\n";
		cont += QString("Language=%1\n").arg(lang->language->selectedItems().at(0)->data(Qt::UserRole+1).toString());
		cont += QString("Composite=%1\n").arg(int(interface->composite->isChecked()));
		cont += QString("TrayIcon=%1\n").arg(int(interface->trayUse->isChecked()));
		cont += QString("Transition=%1\n").arg(interface->transition->value());
		cont += QString("LoopThroghFolder=%1\n").arg(int(interface->loop->isChecked()));
		cont += QString("MenuSensitivity=%1\n").arg(interface->menu->value());
		cont += QString("CloseOnGrey=%1\n").arg(int(interface->grey->isChecked()));
		cont += QString("HideCounter=%1\n").arg(int(interface->hideCounter->isChecked()));
		cont += QString("HideFilename=%1\n").arg(int(interface->hideFilename->isChecked()));
		cont += QString("HideX=%1\n").arg(int(interface->hideX->isChecked()));
		cont += QString("ThumbnailSize=%1\n").arg(thumbnail->thumbSizeSlider->value());

		if(thumbnail->thbPosTop->isChecked())
			cont += "ThumbnailPosition=Top\n";
		else
			cont += "ThumbnailPosition=Bottom\n";

		cont += QString("ThumbnailCache=%1\n").arg(int(thumbnail->thumbCache->isChecked()));
		cont += QString("ThbCacheFile=%1\n").arg(int(thumbnail->cacheFile->isChecked()));

		// These strings minus the "Exif" are the identifier of the labels in tabexif.cpp
		QStringList exifs;
		exifs << "ExifFilename";
		exifs << "ExifFiletype";
		exifs << "ExifFilesize";
		exifs << "ExifDimensions";
		exifs << "ExifMake";
		exifs << "ExifModel";
		exifs << "ExifSoftware";
		exifs << "ExifPhotoTaken";
		exifs << "ExifExposureTime";
		exifs << "ExifFlash";
		exifs << "ExifIso";
		exifs << "ExifSceneType";
		exifs << "ExifFLength";
		exifs << "ExifFNumber";
		exifs << "ExifLightSource";
		exifs << "ExifGps";

		for(int i = 0; i < exifs.length(); ++i) {

			QString temp = exifs.at(i).toLower();
			temp.remove(0,4);
			cont += exifs.at(i) + QString("=%1\n").arg(int(exif->allCheckBoxes[temp]->isChecked()));

		}

		if(exif->exifRotAlw->isChecked())
			cont += "ExifRotation=Always\n";
		else if(exif->exifRotNev->isChecked())
			cont += "ExifRotation=Never\n";
		else if(exif->exifRotAsk->isChecked())
			cont += "ExifRotation=Ask\n";

		cont += "ExifGPSMapService=" + exif->exifGPSBrowser->selectedItems().at(0)->text() + "\n";


		out << cont;
		file.close();

		emit settingsSaved();

	}

}



Settings::~Settings() { }
